@@ -31,7 +31,7 @@
   (def %array-data-vector)
   (def %array-displacement)
   (def %array-displaced-p)
-  (def %array-diplaced-from))
+  (def %array-displaced-from))
 
 (defun %array-rank (array)
   (%array-rank array))
@@ -47,6 +47,11 @@
            (fixnum index))
   (%check-bound array bound index))
 
+(defun check-bound (array bound index)
+  (declare (type index bound)
+           (fixnum index))
+  (%check-bound array bound index))
+
 (defun %with-array-data/fp (array start end)
   (%with-array-data-macro array start end :check-bounds t :check-fill-pointer t))
 
@@ -274,7 +279,7 @@
                        (parse-unknown-type ()
                          (return (result sb!vm:simple-vector-widetag))))))
                (typecase ctype
-                 (union-type
+                 (union-type ; FIXME: forward ref
                   (let ((types (union-type-types ctype)))
                     (cond ((not (every #'numeric-type-p types))
                            (result sb!vm:simple-vector-widetag))
@@ -291,7 +296,7 @@
                            (result sb!vm:simple-array-long-float-widetag))
                           (t
                            (result sb!vm:simple-vector-widetag)))))
-                 (character-set-type
+                 (character-set-type ; FIXME: forward ref
                   #!-sb-unicode (result sb!vm:simple-base-string-widetag)
                   #!+sb-unicode
                   (if (loop for (start . end)
@@ -374,6 +379,24 @@
         (contents-p (values :initial-contents contents))
         (t          (values nil nil))))
 
+(declaim (inline %save-displaced-array-backpointer))
+(defun %save-displaced-array-backpointer (array data)
+  (flet ((purge (pointers)
+           (remove-if (lambda (value)
+                        (or (not value) (eq array value)))
+                      pointers
+                      :key #'weak-pointer-value)))
+    ;; Add backpointer to the new data vector if it has a header.
+    (when (array-header-p data)
+      (setf (%array-displaced-from data)
+            (cons (make-weak-pointer array)
+                  (purge (%array-displaced-from data)))))
+    ;; Remove old backpointer, if any.
+    (let ((old-data (%array-data-vector array)))
+      (when (and (neq data old-data) (array-header-p old-data))
+        (setf (%array-displaced-from old-data)
+              (purge (%array-displaced-from old-data)))))))
+
 ;;; Widetag is the widetag of the underlying vector,
 ;;; it'll be the same as the resulting array widetag only for simple vectors
 (defun %make-array (dimensions widetag n-bits
@@ -481,6 +504,7 @@
   (declare (ignore initial-element
                    initial-contents adjustable
                    fill-pointer displaced-to displaced-index-offset))
+  (declare (explicit-check))
   (multiple-value-bind (widetag n-bits) (%vector-widetag-and-n-bits element-type)
     (apply #'%make-array dimensions widetag n-bits args)))
 
@@ -518,11 +542,11 @@ of specialized arrays is supported."
   ;;
   ;; Allocate and possibly initialize the vector.
   (multiple-value-bind (type n-bits)
-      (sb!impl::%vector-widetag-and-n-bits element-type)
+      (%vector-widetag-and-n-bits element-type)
     (let ((vector
-           (allocate-static-vector type length
-                                   (ceiling (* length n-bits)
-                                            sb!vm:n-word-bits))))
+            (allocate-static-vector type length
+                                    (ceiling (* length n-bits)
+                                             sb!vm:n-word-bits))))
       (cond (initial-element-p
              (fill vector initial-element))
             (initial-contents-p
@@ -594,7 +618,7 @@ of specialized arrays is supported."
 
 ;;; Like DOVECTOR, but more magical -- can't use this on host.
 (defmacro do-vector-data ((elt vector &optional result) &body body)
-  (multiple-value-bind (forms decls) (parse-body body :doc-string-allowed nil)
+  (multiple-value-bind (forms decls) (parse-body body nil)
     (with-unique-names (index vec start end ref)
       `(with-array-data ((,vec ,vector)
                          (,start)
@@ -617,6 +641,7 @@ of specialized arrays is supported."
                                   extra-params check-bounds)
              `(progn
                 (defun ,accessor-name (array index ,@extra-params)
+                  (declare (explicit-check))
                   (declare (optimize speed
                                      ;; (SAFETY 0) is ok. All calls to
                                      ;; these functions are generated by
@@ -652,11 +677,11 @@ of specialized arrays is supported."
   (define hairy-data-vector-ref/check-bounds
       slow-hairy-data-vector-ref/check-bounds
     !find-data-vector-reffer/check-bounds
-    nil (%check-bound array (array-dimension array 0)))
+    nil (check-bound array (array-dimension array 0)))
   (define hairy-data-vector-set/check-bounds
       slow-hairy-data-vector-set/check-bounds
     !find-data-vector-setter/check-bounds
-    (new-value) (%check-bound array (array-dimension array 0))))
+    (new-value) (check-bound array (array-dimension array 0))))
 
 (defun hairy-ref-error (array index &optional new-value)
   (declare (ignore index new-value))
@@ -667,8 +692,11 @@ of specialized arrays is supported."
 (macrolet ((define-reffer (saetp check-form)
              (let* ((type (sb!vm:saetp-specifier saetp))
                     (atype `(simple-array ,type (*))))
-               `(named-lambda optimized-data-vector-ref (vector index)
+               `(named-lambda (optimized-data-vector-ref ,type) (vector index)
                   (declare (optimize speed (safety 0))
+                           ;; Obviously these all coerce raw words to lispobjs
+                           ;; so don't keep spewing notes about it.
+                           (muffle-conditions compiler-note)
                            (ignorable index))
                   ,(if type
                        `(data-vector-ref (the ,atype vector)
@@ -680,8 +708,11 @@ of specialized arrays is supported."
            (define-setter (saetp check-form)
              (let* ((type (sb!vm:saetp-specifier saetp))
                     (atype `(simple-array ,type (*))))
-               `(named-lambda optimized-data-vector-set (vector index new-value)
+               `(named-lambda (optimized-data-vector-set ,type) (vector index new-value)
                   (declare (optimize speed (safety 0)))
+                  ;; Impossibly setting an elt of an (ARRAY NIL)
+                  ;; returns no value. And nobody cares.
+                  (declare (muffle-conditions compiler-note))
                   (data-vector-set (the ,atype vector)
                                    (locally
                                        (declare (optimize (safety 1)))
@@ -726,19 +757,21 @@ of specialized arrays is supported."
       (progn)
       #'slow-hairy-data-vector-set)
     (define-reffers %%data-vector-reffers/check-bounds%% define-reffer
-      (%check-bound vector (length vector))
+      (check-bound vector (length vector))
       #'slow-hairy-data-vector-ref/check-bounds)
     (define-reffers %%data-vector-setters/check-bounds%% define-setter
-      (%check-bound vector (length vector))
+      (check-bound vector (length vector))
       #'slow-hairy-data-vector-set/check-bounds)))
 
 ;;; (Ordinary DATA-VECTOR-REF usage compiles into a vop, but
 ;;; DATA-VECTOR-REF is also FOLDABLE, and this ordinary function
 ;;; definition is needed for the compiler to use in constant folding.)
 (defun data-vector-ref (array index)
+  (declare (explicit-check))
   (hairy-data-vector-ref array index))
 
 (defun data-vector-ref-with-offset (array index offset)
+  (declare (explicit-check))
   (hairy-data-vector-ref array (+ index offset)))
 
 (defun invalid-array-p (array)
@@ -758,7 +791,7 @@ of specialized arrays is supported."
                `(array ,et ,dims)
                `(vector ,et ,@dims)))))
 
-(declaim (ftype (function (array integer integer &optional t) nil)
+(declaim (ftype (function (array t integer &optional t) nil)
                 invalid-array-index-error))
 (defun invalid-array-index-error (array index bound &optional axis)
   (if (invalid-array-p array)
@@ -770,53 +803,71 @@ of specialized arrays is supported."
              :expected-type `(integer 0 (,bound)))))
 
 ;;; SUBSCRIPTS has a dynamic-extent list structure and is destroyed
-(defun %array-row-major-index (array subscripts
-                                     &optional (invalid-index-error-p t))
-  (declare (array array)
-           (list subscripts))
-  (let ((rank (array-rank array)))
-    (unless (= rank (length subscripts))
-      (error "wrong number of subscripts, ~W, for array of rank ~W"
-             (length subscripts) rank))
-    (if (array-header-p array)
-        (do ((subs (nreverse subscripts) (cdr subs))
-             (axis (1- (array-rank array)) (1- axis))
-             (chunk-size 1)
-             (result 0))
-            ((null subs) result)
-          (declare (list subs) (fixnum axis chunk-size result))
-          (let ((index (car subs))
-                (dim (%array-dimension array axis)))
-            (declare (fixnum dim))
-            (unless (and (fixnump index) (< -1 index dim))
-              (if invalid-index-error-p
-                  (invalid-array-index-error array index dim axis)
-                  (return-from %array-row-major-index nil)))
-            (incf result (* chunk-size (the fixnum index)))
-            (setf chunk-size (* chunk-size dim))))
-        (let ((index (first subscripts))
-              (length (length (the (simple-array * (*)) array))))
-          (unless (and (fixnump index) (< -1 index length))
-            (if invalid-index-error-p
-                (invalid-array-index-error array index length)
-                (return-from %array-row-major-index nil)))
-          index))))
+(defun %array-row-major-index (array &rest subscripts)
+  (declare (truly-dynamic-extent subscripts)
+           (array array))
+  (let ((length (length subscripts)))
+    (cond ((array-header-p array)
+           (let ((rank (%array-rank array)))
+             (unless (= rank length)
+               (error "wrong number of subscripts, ~W, for array of rank ~W."
+                      length rank))
+             (do ((axis (1- rank) (1- axis))
+                  (chunk-size 1)
+                  (result 0))
+                 ((minusp axis) result)
+               (declare (fixnum axis chunk-size result))
+               (let ((index (fast-&rest-nth axis subscripts))
+                     (dim (%array-dimension array axis)))
+                 (unless (and (fixnump index) (< -1 index dim))
+                   (invalid-array-index-error array index dim axis))
+                 (setf result
+                       (truly-the fixnum
+                                  (+ result
+                                     (truly-the fixnum (* chunk-size index))))
+                       chunk-size (truly-the fixnum (* chunk-size dim)))))))
+          ((/= length 1)
+           (error "Wrong number of subscripts, ~W, for array of rank 1."
+                  length))
+          (t
+           (let ((index (fast-&rest-nth 0 subscripts))
+                 (length (length (the (simple-array * (*)) array))))
+             (unless (and (fixnump index) (< -1 index length))
+               (invalid-array-index-error array index length))
+             index)))))
 
 (defun array-in-bounds-p (array &rest subscripts)
   #!+sb-doc
   "Return T if the SUBSCRIPTS are in bounds for the ARRAY, NIL otherwise."
-  (if (%array-row-major-index array subscripts nil)
-      t))
+  (declare (truly-dynamic-extent subscripts))
+  (let ((length (length subscripts)))
+    (cond ((array-header-p array)
+           (let ((rank (%array-rank array)))
+             (unless (= rank length)
+               (error "Wrong number of subscripts, ~W, for array of rank ~W."
+                      length rank))
+             (loop for i below length
+                   for s = (fast-&rest-nth i subscripts)
+                   always (and (typep s '(and fixnum unsigned-byte))
+                               (< s (%array-dimension array i))))))
+          ((/= length 1)
+           (error "Wrong number of subscripts, ~W, for array of rank 1."
+                  length))
+          (t
+           (let ((subscript (fast-&rest-nth 0 subscripts)))
+             (and (typep subscript '(and fixnum unsigned-byte))
+                  (< subscript
+                     (length (truly-the (simple-array * (*)) array)))))))))
 
 (defun array-row-major-index (array &rest subscripts)
   (declare (truly-dynamic-extent subscripts))
-  (%array-row-major-index array subscripts))
+  (apply #'%array-row-major-index array subscripts))
 
 (defun aref (array &rest subscripts)
   #!+sb-doc
   "Return the element of the ARRAY specified by the SUBSCRIPTS."
   (declare (truly-dynamic-extent subscripts))
-  (row-major-aref array (%array-row-major-index array subscripts)))
+  (row-major-aref array (apply #'%array-row-major-index array subscripts)))
 
 ;;; (setf aref/bit/sbit) are implemented using setf-functions,
 ;;; because they have to work with (setf (apply #'aref array subscripts))
@@ -826,7 +877,7 @@ of specialized arrays is supported."
 (defun (setf aref) (new-value array &rest subscripts)
   (declare (truly-dynamic-extent subscripts)
            (type array array))
-  (setf (row-major-aref array (%array-row-major-index array subscripts))
+  (setf (row-major-aref array (apply #'%array-row-major-index array subscripts))
         new-value))
 
 (defun row-major-aref (array index)
@@ -856,7 +907,7 @@ of specialized arrays is supported."
   (declare (type (array bit) bit-array)
            (truly-dynamic-extent subscripts)
            (optimize (safety 1)))
-  (row-major-aref bit-array (%array-row-major-index bit-array subscripts)))
+  (row-major-aref bit-array (apply #'%array-row-major-index bit-array subscripts)))
 
 (defun (setf bit) (new-value bit-array &rest subscripts)
   (declare (type (array bit) bit-array)
@@ -864,7 +915,7 @@ of specialized arrays is supported."
            (truly-dynamic-extent subscripts)
            (optimize (safety 1)))
   (setf (row-major-aref bit-array
-                        (%array-row-major-index bit-array subscripts))
+                        (apply #'%array-row-major-index bit-array subscripts))
         new-value))
 
 (defun sbit (simple-bit-array &rest subscripts)
@@ -874,7 +925,7 @@ of specialized arrays is supported."
            (truly-dynamic-extent subscripts)
            (optimize (safety 1)))
   (row-major-aref simple-bit-array
-                  (%array-row-major-index simple-bit-array subscripts)))
+                  (apply #'%array-row-major-index simple-bit-array subscripts)))
 
 (defun (setf sbit) (new-value bit-array &rest subscripts)
   (declare (type (simple-array bit) bit-array)
@@ -882,7 +933,7 @@ of specialized arrays is supported."
            (truly-dynamic-extent subscripts)
            (optimize (safety 1)))
   (setf (row-major-aref bit-array
-                        (%array-row-major-index bit-array subscripts))
+                        (apply #'%array-row-major-index bit-array subscripts))
         new-value))
 
 ;;;; miscellaneous array properties
@@ -890,39 +941,27 @@ of specialized arrays is supported."
 (defun array-element-type (array)
   #!+sb-doc
   "Return the type of the elements of the array"
-  (let ((widetag (%other-pointer-widetag array)))
-    (macrolet ((pick-element-type (&rest stuff)
-                 `(cond ,@(mapcar (lambda (stuff)
-                                    (cons
-                                     (let ((item (car stuff)))
-                                       (cond ((eq item t)
-                                              t)
-                                             ((listp item)
-                                              (cons 'or
-                                                    (mapcar (lambda (x)
-                                                              `(= widetag ,x))
-                                                            item)))
-                                             (t
-                                              `(= widetag ,item))))
-                                     (cdr stuff)))
-                                  stuff))))
-      #.`(pick-element-type
-          ,@(map 'list
-                 (lambda (saetp)
-                   `(,(if (sb!vm:saetp-complex-typecode saetp)
-                          (list (sb!vm:saetp-typecode saetp)
-                                (sb!vm:saetp-complex-typecode saetp))
-                          (sb!vm:saetp-typecode saetp))
-                     ',(sb!vm:saetp-specifier saetp)))
-                 sb!vm:*specialized-array-element-type-properties*)
-          ((sb!vm:simple-array-widetag
-            sb!vm:complex-vector-widetag
-            sb!vm:complex-array-widetag)
-           (with-array-data ((array array) (start) (end))
-             (declare (ignore start end))
-             (array-element-type array)))
-          (t
-           (error 'type-error :datum array :expected-type 'array))))))
+  (let ((widetag (%other-pointer-widetag array))
+        (table (load-time-value
+                (let ((table (make-array 256 :initial-element nil)))
+                  (dotimes (i (length sb!vm:*specialized-array-element-type-properties*) table)
+                    (let* ((saetp (aref sb!vm:*specialized-array-element-type-properties* i))
+                           (typecode (sb!vm:saetp-typecode saetp))
+                           (complex-typecode (sb!vm:saetp-complex-typecode saetp))
+                           (specifier (sb!vm:saetp-specifier saetp)))
+                      (aver (typep specifier '(or list symbol)))
+                      (setf (aref table typecode) specifier)
+                      (when complex-typecode
+                        (setf (aref table complex-typecode) specifier)))))
+                t)))
+    (let ((result (aref table widetag)))
+      (if result
+          (truly-the (or list symbol) result)
+          ;; (MAKE-ARRAY :ELEMENT-TYPE NIL) goes to this branch, but
+          ;; gets the right answer in the end
+          (with-array-data ((array array) (start) (end))
+            (declare (ignore start end))
+            (truly-the (or list symbol) (aref table (%other-pointer-widetag array))))))))
 
 (defun array-rank (array)
   #!+sb-doc
@@ -1013,11 +1052,13 @@ of specialized arrays is supported."
 (defun fill-pointer (vector)
   #!+sb-doc
   "Return the FILL-POINTER of the given VECTOR."
+  (declare (explicit-check))
   (if (array-has-fill-pointer-p vector)
       (%array-fill-pointer vector)
       (fill-pointer-error vector)))
 
 (defun %set-fill-pointer (vector new)
+  (declare (explicit-check))
   (flet ((oops (x)
            (fill-pointer-error vector x)))
     (if (array-has-fill-pointer-p vector)
@@ -1037,6 +1078,7 @@ of specialized arrays is supported."
    to NEW-ELEMENT, and increment the fill pointer by one. If the fill pointer is
    too large, NIL is returned, otherwise the index of the pushed element is
    returned."
+  (declare (explicit-check))
   (let ((fill-pointer (fill-pointer array)))
     (declare (fixnum fill-pointer))
     (cond ((= fill-pointer (%array-available-elements array))
@@ -1049,6 +1091,7 @@ of specialized arrays is supported."
 
 (defun vector-push-extend (new-element vector &optional min-extension)
   (declare (type (or null fixnum) min-extension))
+  (declare (explicit-check))
   (let ((fill-pointer (fill-pointer vector)))
     (declare (fixnum fill-pointer))
     (when (= fill-pointer (%array-available-elements vector))
@@ -1068,6 +1111,7 @@ of specialized arrays is supported."
   #!+sb-doc
   "Decrease the fill pointer by 1 and return the element pointed to by the
   new fill pointer."
+  (declare (explicit-check))
   (let ((fill-pointer (fill-pointer array)))
     (declare (fixnum fill-pointer))
     (if (zerop fill-pointer)
@@ -1091,7 +1135,7 @@ of specialized arrays is supported."
   "Adjust ARRAY's dimensions to the given DIMENSIONS and stuff."
   (when (invalid-array-p array)
     (invalid-array-error array))
-  (binding* ((dimensions (if (listp dimensions) dimensions (list dimensions)))
+  (binding* ((dimensions (ensure-list dimensions))
              (array-rank (array-rank array))
              (()
               (unless (= (length dimensions) array-rank)
@@ -1134,7 +1178,8 @@ of specialized arrays is supported."
                        array-data))))
           (displaced-to
              ;; We already established that no INITIAL-CONTENTS was supplied.
-             (unless (subtypep element-type (array-element-type displaced-to))
+             (unless (or (eql element-type (array-element-type displaced-to))
+                         (subtypep element-type (array-element-type displaced-to)))
                ;; See lp#1331299 again. Require exact match on upgraded type?
                (error "can't displace an array of type ~S into another of ~
                        type ~S"
@@ -1309,73 +1354,55 @@ of specialized arrays is supported."
 ;;; corruption, but because it walks the chain *upwards*, which
 ;;; may violate user expectations.
 
-(defun %save-displaced-array-backpointer (array data)
-  (flet ((purge (pointers)
-           (remove-if (lambda (value)
-                        (or (not value) (eq array value)))
-                      pointers
-                      :key #'weak-pointer-value)))
-    ;; Add backpointer to the new data vector if it has a header.
-    (when (array-header-p data)
-      (setf (%array-displaced-from data)
-            (cons (make-weak-pointer array)
-                  (purge (%array-displaced-from data)))))
-    ;; Remove old backpointer, if any.
-    (let ((old-data (%array-data-vector array)))
-      (when (and (neq data old-data) (array-header-p old-data))
-        (setf (%array-displaced-from old-data)
-              (purge (%array-displaced-from old-data)))))))
-
-(defun %walk-displaced-array-backpointers (array new-length)
-  (dolist (p (%array-displaced-from array))
-    (let ((from (weak-pointer-value p)))
-      (when (and from (eq array (%array-data-vector from)))
-        (let ((requires (+ (%array-available-elements from)
-                           (%array-displacement from))))
-          (unless (>= new-length requires)
-            ;; ANSI sayeth (ADJUST-ARRAY dictionary entry):
-            ;;
-            ;;   "If A is displaced to B, the consequences are unspecified if B is
-            ;;   adjusted in such a way that it no longer has enough elements to
-            ;;   satisfy A.
-            ;;
-            ;; since we're hanging on a weak pointer here, we can't signal an
-            ;; error right now: the array that we're looking at might be
-            ;; garbage. Instead, we set all dimensions to zero so that next
-            ;; safe access to the displaced array will trap. Additionally, we
-            ;; save the original dimensions, so we can signal a more
-            ;; understandable error when the time comes.
-            (%walk-displaced-array-backpointers from 0)
-            (setf (%array-fill-pointer from) 0
-                  (%array-available-elements from) 0
-                  (%array-displaced-p from) (array-dimensions array))
-            (dotimes (i (%array-rank from))
-              (setf (%array-dimension from i) 0))))))))
-
 ;;; Fill in array header with the provided information, and return the array.
 (defun set-array-header (array data length fill-pointer displacement dimensions
                          displacedp newp)
-  (if newp
-      (setf (%array-displaced-from array) nil)
-      (%walk-displaced-array-backpointers array length))
-  (when displacedp
-    (%save-displaced-array-backpointer array data))
-  (setf (%array-data-vector array) data)
-  (setf (%array-available-elements array) length)
-  (cond (fill-pointer
-         (setf (%array-fill-pointer array) fill-pointer)
-         (setf (%array-fill-pointer-p array) t))
-        (t
-         (setf (%array-fill-pointer array) length)
-         (setf (%array-fill-pointer-p array) nil)))
-  (setf (%array-displacement array) displacement)
-  (if (listp dimensions)
-      (dotimes (axis (array-rank array))
-        (declare (type index axis))
-        (setf (%array-dimension array axis) (pop dimensions)))
-      (setf (%array-dimension array 0) dimensions))
-  (setf (%array-displaced-p array) displacedp)
-  array)
+  (labels ((%walk-displaced-array-backpointers (array new-length)
+             (dolist (p (%array-displaced-from array))
+               (let ((from (weak-pointer-value p)))
+                 (when (and from (eq array (%array-data-vector from)))
+                   (let ((requires (+ (%array-available-elements from)
+                                      (%array-displacement from))))
+                     (unless (>= new-length requires)
+                       ;; ANSI sayeth (ADJUST-ARRAY dictionary entry):
+                       ;;
+                       ;;   "If A is displaced to B, the consequences are unspecified if B is
+                       ;;   adjusted in such a way that it no longer has enough elements to
+                       ;;   satisfy A.
+                       ;;
+                       ;; since we're hanging on a weak pointer here, we can't signal an
+                       ;; error right now: the array that we're looking at might be
+                       ;; garbage. Instead, we set all dimensions to zero so that next
+                       ;; safe access to the displaced array will trap. Additionally, we
+                       ;; save the original dimensions, so we can signal a more
+                       ;; understandable error when the time comes.
+                       (%walk-displaced-array-backpointers from 0)
+                       (setf (%array-fill-pointer from) 0
+                             (%array-available-elements from) 0
+                             (%array-displaced-p from) (array-dimensions array))
+                       (dotimes (i (%array-rank from))
+                         (setf (%array-dimension from i) 0)))))))))
+    (if newp
+        (setf (%array-displaced-from array) nil)
+        (%walk-displaced-array-backpointers array length))
+    (when displacedp
+      (%save-displaced-array-backpointer array data))
+    (setf (%array-data-vector array) data)
+    (setf (%array-available-elements array) length)
+    (cond (fill-pointer
+           (setf (%array-fill-pointer array) fill-pointer)
+           (setf (%array-fill-pointer-p array) t))
+          (t
+           (setf (%array-fill-pointer array) length)
+           (setf (%array-fill-pointer-p array) nil)))
+    (setf (%array-displacement array) displacement)
+    (if (listp dimensions)
+        (dotimes (axis (array-rank array))
+          (declare (type index axis))
+          (setf (%array-dimension array axis) (pop dimensions)))
+        (setf (%array-dimension array 0) dimensions))
+    (setf (%array-displaced-p array) displacedp)
+    array))
 
 ;;; User visible extension
 (declaim (ftype (function (array) (values (simple-array * (*)) &optional))
