@@ -80,6 +80,7 @@
 (defstruct (negation-type (:include ctype
                                     (class-info (type-class-or-lose 'negation)))
                           (:copier nil)
+                          (:constructor make-negation-type (type))
                           #!+cmu (:pure nil))
   (type (missing-arg) :type ctype :read-only t))
 
@@ -283,56 +284,6 @@
                        (:copier nil))
   (name nil :type symbol :read-only t))
 
-;;; a list of all the float "formats" (i.e. internal representations;
-;;; nothing to do with #'FORMAT), in order of decreasing precision
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (defparameter *float-formats*
-    '(long-float double-float single-float short-float)))
-
-;;; The type of a float format.
-(deftype float-format () `(member ,@*float-formats*))
-
-;;; A NUMERIC-TYPE represents any numeric type, including things
-;;; such as FIXNUM.
-(defstruct (numeric-type (:include ctype
-                                   (class-info (type-class-or-lose 'number)))
-                         (:constructor %make-numeric-type)
-                         (:copier nil))
-  ;; Formerly defined in every CTYPE, but now just in the ones
-  ;; for which enumerability is variable.
-  (enumerable nil :read-only t)
-  ;; the kind of numeric type we have, or NIL if not specified (just
-  ;; NUMBER or COMPLEX)
-  ;;
-  ;; KLUDGE: A slot named CLASS for a non-CLASS value is bad.
-  ;; Especially when a CLASS value *is* stored in another slot (called
-  ;; CLASS-INFO:-). Perhaps this should be called CLASS-NAME? Also
-  ;; weird that comment above says "Numeric-Type is used to represent
-  ;; all numeric types" but this slot doesn't allow COMPLEX as an
-  ;; option.. how does this fall into "not specified" NIL case above?
-  ;; Perhaps someday we can switch to CLOS and make NUMERIC-TYPE
-  ;; be an abstract base class and INTEGER-TYPE, RATIONAL-TYPE, and
-  ;; whatnot be concrete subclasses..
-  (class nil :type (member integer rational float nil) :read-only t)
-  ;; "format" for a float type (i.e. type specifier for a CPU
-  ;; representation of floating point, e.g. 'SINGLE-FLOAT -- nothing
-  ;; to do with #'FORMAT), or NIL if not specified or not a float.
-  ;; Formats which don't exist in a given implementation don't appear
-  ;; here.
-  (format nil :type (or float-format null) :read-only t)
-  ;; Is this a complex numeric type?  Null if unknown (only in NUMBER).
-  ;;
-  ;; FIXME: I'm bewildered by FOO-P names for things not intended to
-  ;; interpreted as truth values. Perhaps rename this COMPLEXNESS?
-  (complexp :real :type (member :real :complex nil) :read-only t)
-  ;; The upper and lower bounds on the value, or NIL if there is no
-  ;; bound. If a list of a number, the bound is exclusive. Integer
-  ;; types never have exclusive bounds, i.e. they may have them on
-  ;; input, but they're canonicalized to inclusive bounds before we
-  ;; store them here.
-  (low nil :type (or number cons null) :read-only t)
-  (high nil :type (or number cons null) :read-only t))
-
 ;; For some numeric subtypes, uniqueness of the object representation
 ;; is enforced. These encompass all array specializations and more.
 (defglobal *unsigned-byte-type* -1)
@@ -487,13 +438,6 @@
                      :high high
                      :enumerable enumerable))
 
-(defstruct (character-set-type
-            (:include ctype
-                      (class-info (type-class-or-lose 'character-set)))
-            (:constructor %make-character-set-type (pairs))
-            (:copier nil))
-  (pairs (missing-arg) :type list :read-only t))
-
 ;; Interned character-set types.
 (defglobal *character-type* -1)
 #!+sb-unicode
@@ -509,10 +453,10 @@
             (%make-character-set-type (list (cons low high))))))
     (setq *character-type* (range 0 (1- sb!xc:char-code-limit)))
     #!+sb-unicode
-    (setq *base-char-type* (range 0 127)
-          *extended-char-type* (range 128 (1- sb!xc:char-code-limit)))))
+    (setq *base-char-type* (range 0 (1- base-char-code-limit))
+          *extended-char-type* (range base-char-code-limit (1- sb!xc:char-code-limit)))))
 
-(defun make-character-set-type (&key pairs)
+(defun make-character-set-type (pairs)
   ; (aver (equal (mapcar #'car pairs)
   ;              (sort (mapcar #'car pairs) #'<)))
   ;; aver that the cars of the list elements are sorted into increasing order
@@ -539,95 +483,69 @@
         *empty-type*
         (or (and (singleton-p pairs)
                  (let* ((pair (car pairs))
-                        (low (car pair)))
-                   (case (cdr pair) ; high
+                        (low (car pair))
+                        (high (cdr pair)))
+                   (case high
                      (#.(1- sb!xc:char-code-limit)
                       (case low
                         (0 *character-type*)
-                        #!+sb-unicode (128 *extended-char-type*)))
+                        #!+sb-unicode
+                        (#.base-char-code-limit *extended-char-type*)))
                      #!+sb-unicode
-                     (127 (if (eql low 0) *base-char-type*)))))
+                     (#.(1- base-char-code-limit)
+                      (when (eql low 0)
+                        *base-char-type*)))))
             (%make-character-set-type pairs)))))
 
-;;; An ARRAY-TYPE is used to represent any array type, including
-;;; things such as SIMPLE-BASE-STRING.
-(defstruct (array-type (:include ctype
-                                 (class-info (type-class-or-lose 'array)))
-                       (:constructor %make-array-type
-                        (dimensions complexp element-type
-                                    specialized-element-type))
-                       (:copier nil))
-  ;; the dimensions of the array, or * if unspecified. If a dimension
-  ;; is unspecified, it is *.
-  (dimensions '* :type (or list (member *)) :read-only t)
-  ;; Is this not a simple array type? (:MAYBE means that we don't know.)
-  (complexp :maybe :type (member t nil :maybe) :read-only t)
-  ;; the element type as originally specified
-  (element-type (missing-arg) :type ctype :read-only t)
-  ;; the element type as it is specialized in this implementation
-  (specialized-element-type *wild-type* :type ctype :read-only t))
-
 ;; For all ctypes which are the element types of specialized arrays,
 ;; 3 ctype objects are stored for the rank-1 arrays of that specialization,
 ;; one for each of simple, maybe-simple, and non-simple (in that order),
 ;; and 2 ctype objects for unknown-rank arrays, one each for simple
 ;; and maybe-simple. (Unknown rank, known-non-simple isn't important)
 (defglobal *canonical-array-ctypes* -1)
-(defconstant +canon-array-ctype-hash-divisor+ 37) ; arbitrary-ish
 (defun !intern-important-array-type-instances ()
   ;; Having made the canonical numeric and character ctypes
   ;; representing the points in the type lattice for which there
   ;; are array specializations, we can make the canonical array types.
-  (let* ((element-types
-          (list*
-           *universal-type* *wild-type* *empty-type*
-           *character-type*
-           #!+sb-unicode *base-char-type*
-           ;; FIXME: This one is can't be used by MAKE-ARRAY-TYPE?
-           #!+sb-unicode *extended-char-type*
-           *real-ffloat-type* *complex-ffloat-type*
-           *real-dfloat-type* *complex-dfloat-type*
-           (delete
-            nil
-            ;; Possibly could use the SAETP-IMPORTANCE as sort criterion
-            ;; so that collisions in a bucket place the more important
-            ;; array type first.
-            (mapcar
-             (lambda (x)
-               (cond ((typep x '(cons (eql unsigned-byte)))
-                      (aref *unsigned-byte-n-types* (cadr x)))
-                     ((eq x 'bit)
-                      (aref *unsigned-byte-n-types* 1))
-                     ((typep x '(cons (eql signed-byte)))
-                      ;; 1- because there is no such thing as (signed-byte 0)
-                      (aref *signed-byte-n-types* (1- (cadr x))))
-                     ;; FIXNUM is its own thing, why? See comment in vm-array
-                     ;; saying to "See the comment in PRIMITIVE-TYPE-AUX"
-                     ((eq x 'fixnum) ; One good kludge deserves another.
-                      (aref *signed-byte-n-types* (1- sb!vm:n-fixnum-bits)))))
-             '#.*specialized-array-element-types*))))
-         (n (length element-types))
-         (data-vector (make-array (* 5 n)))
-         (index 0)
-         (hashtable (make-array +canon-array-ctype-hash-divisor+
-                                :initial-element nil)))
-    ;; This is a compact binned table. A full-blown hashtable is unneeded.
-    #-sb-xc (aver (< (/ n (length hashtable)) 80/100)) ; assert reasonable load
-    (flet ((make-it (dims complexp type)
-             (setf (aref data-vector (prog1 index (incf index)))
-                   (mark-ctype-interned
-                    (%make-array-type dims complexp type type)))))
-      (dolist (element-type element-types)
-        (let ((bin (mod (type-hash-value element-type)
-                        +canon-array-ctype-hash-divisor+)))
-          (setf (aref hashtable bin)
-                (nconc (aref hashtable bin) (list (cons element-type index))))
-          (make-it '(*) nil    element-type)
-          (make-it '(*) :maybe element-type)
-          (make-it '(*) t      element-type)
-          (make-it '*   nil    element-type)
-          (make-it '*   :maybe element-type))))
-    (setq *canonical-array-ctypes* (cons data-vector hashtable))))
+  (setq *canonical-array-ctypes* (make-array (* 32 5)))
+  (labels ((make-1 (type-index dims complexp type)
+             (setf (!ctype-saetp-index type) type-index)
+             (mark-ctype-interned (%make-array-type dims complexp type type)))
+           (make-all (element-type type-index)
+             (replace *canonical-array-ctypes*
+                      (list (make-1 type-index '(*) nil    element-type)
+                            (make-1 type-index '(*) :maybe element-type)
+                            (make-1 type-index '(*) t      element-type)
+                            (make-1 type-index '*   nil    element-type)
+                            (make-1 type-index '*   :maybe element-type))
+                      :start1 (* type-index 5))))
+    (let ((index 0))
+      (dolist (x '#.*specialized-array-element-types*)
+        (make-all
+         (cond ((typep x '(cons (eql unsigned-byte)))
+                (aref *unsigned-byte-n-types* (cadr x)))
+               ((eq x 'bit) (aref *unsigned-byte-n-types* 1))
+               ((typep x '(cons (eql signed-byte)))
+                ;; 1- because there is no such thing as (signed-byte 0)
+                (aref *signed-byte-n-types* (1- (cadr x))))
+               ;; FIXNUM is its own thing, why? See comment in vm-array
+               ;; saying to "See the comment in PRIMITIVE-TYPE-AUX"
+               ((eq x 'fixnum) ; One good kludge deserves another.
+                (aref *signed-byte-n-types* (1- sb!vm:n-fixnum-bits)))
+               ((eq x 'single-float) *real-ffloat-type*)
+               ((eq x 'double-float) *real-dfloat-type*)
+               ((equal x '(complex single-float)) *complex-ffloat-type*)
+               ((equal x '(complex double-float)) *complex-dfloat-type*)
+               ((eq x 'character) *character-type*)
+               #!+sb-unicode ((eq x 'base-char) *base-char-type*)
+               ((eq x t) *universal-type*)
+               ((null x) *empty-type*))
+         index)
+        (incf index))
+      ;; Index 31 is available to store *WILD-TYPE*
+      ;; because there are fewer than 32 array widetags.
+      (aver (< index 31))
+      (make-all *wild-type* 31))))
 
 (declaim (ftype (sfunction (t &key (:complexp t)
                                    (:element-type t)
@@ -635,21 +553,15 @@
                            ctype) make-array-type))
 (defun make-array-type (dimensions &key (complexp :maybe) element-type
                                         (specialized-element-type *wild-type*))
-  (or (and (eq element-type specialized-element-type)
+  (if (and (eq element-type specialized-element-type)
            (or (and (eq dimensions '*) (neq complexp t))
-               (typep dimensions '(cons (eql *) null)))
-           (let ((table *canonical-array-ctypes*))
-             (dolist (cell (svref (cdr table)
-                                  (mod (type-hash-value element-type)
-                                       +canon-array-ctype-hash-divisor+)))
-               (when (eq (car cell) element-type)
-                 (return
-                  (truly-the ctype
-                   (svref (car table)
-                          (+ (cdr cell)
-                             (if (listp dimensions) 0 3)
-                             (ecase complexp
-                              ((nil) 0) ((:maybe) 1) ((t) 2))))))))))
+               (typep dimensions '(cons (eql *) null))))
+      (let ((res (svref *canonical-array-ctypes*
+                        (+ (* (!ctype-saetp-index element-type) 5)
+                           (if (listp dimensions) 0 3)
+                           (ecase complexp ((nil) 0) ((:maybe) 1) ((t) 2))))))
+        (aver (eq (array-type-element-type res) element-type))
+        res)
       (%make-array-type dimensions
                         complexp element-type specialized-element-type)))
 
@@ -768,57 +680,6 @@
   (append (member-type-fp-zeroes type)
           (xset-members (member-type-xset type))))
 
-;;; A COMPOUND-TYPE is a type defined out of a set of types, the
-;;; common parent of UNION-TYPE and INTERSECTION-TYPE.
-(defstruct (compound-type (:include ctype)
-                          (:constructor nil)
-                          (:copier nil))
-  ;; Formerly defined in every CTYPE, but now just in the ones
-  ;; for which enumerability is variable.
-  (enumerable nil :read-only t)
-  (types nil :type list :read-only t))
-
-;;; A UNION-TYPE represents a use of the OR type specifier which we
-;;; couldn't canonicalize to something simpler. Canonical form:
-;;;   1. All possible pairwise simplifications (using the UNION2 type
-;;;      methods) have been performed. Thus e.g. there is never more
-;;;      than one MEMBER-TYPE component. FIXME: As of sbcl-0.6.11.13,
-;;;      this hadn't been fully implemented yet.
-;;;   2. There are never any UNION-TYPE components.
-;;;
-;;; TODO: As STRING is an especially important union type,
-;;; it could be interned by canonicalizing its subparts into
-;;; ARRAY of {CHARACTER,BASE-CHAR,NIL} in that exact order always.
-;;; It will therefore admit quick TYPE=, but not quick failure, since
-;;;   (type= (specifier-type '(or (simple-array (member #\a) (*))
-;;;                               (simple-array character (*))
-;;;                               (simple-array nil (*))))
-;;;          (specifier-type 'simple-string)) => T and T
-;;; even though (MEMBER #\A) is not TYPE= to BASE-CHAR.
-;;;
-(defstruct (union-type (:include compound-type
-                                 (class-info (type-class-or-lose 'union)))
-                       (:constructor make-union-type (enumerable types))
-                       (:copier nil)))
-
-;;; An INTERSECTION-TYPE represents a use of the AND type specifier
-;;; which we couldn't canonicalize to something simpler. Canonical form:
-;;;   1. All possible pairwise simplifications (using the INTERSECTION2
-;;;      type methods) have been performed. Thus e.g. there is never more
-;;;      than one MEMBER-TYPE component.
-;;;   2. There are never any INTERSECTION-TYPE components: we've
-;;;      flattened everything into a single INTERSECTION-TYPE object.
-;;;   3. There are never any UNION-TYPE components. Either we should
-;;;      use the distributive rule to rearrange things so that
-;;;      unions contain intersections and not vice versa, or we
-;;;      should just punt to using a HAIRY-TYPE.
-(defstruct (intersection-type (:include compound-type
-                                        (class-info (type-class-or-lose
-                                                     'intersection)))
-                              (:constructor %make-intersection-type
-                                            (enumerable types))
-                              (:copier nil)))
-
 ;;; Return TYPE converted to canonical form for a situation where the
 ;;; "type" '* (which SBCL still represents as a type even though ANSI
 ;;; CL defines it as a related but different kind of placeholder) is
@@ -828,16 +689,6 @@
       *universal-type*
       type))
 
-;;; A CONS-TYPE is used to represent a CONS type.
-(defstruct (cons-type (:include ctype (class-info (type-class-or-lose 'cons)))
-                      (:constructor
-                       %make-cons-type (car-type
-                                        cdr-type))
-                      (:copier nil))
-  ;; the CAR and CDR element types (to support ANSI (CONS FOO BAR) types)
-  (car-type (missing-arg) :type ctype :read-only t)
-  (cdr-type (missing-arg) :type ctype :read-only t))
-
 ;; The function caches work significantly better when there
 ;; is a unique object that stands for the specifier (CONS T T).
 (defglobal *cons-t-t-type* -1)
@@ -918,7 +769,7 @@
 ;;; The xc host uses an ordinary hash table for memoization.
 #+sb-xc-host
 (let ((table (make-hash-table :test 'equal)))
-  (defun values-specifier-type-memo-wrapper (thunk specifier)
+  (defun !values-specifier-type-memo-wrapper (thunk specifier)
     (multiple-value-bind (type yesp) (gethash specifier table)
       (if yesp
           type
@@ -953,6 +804,7 @@
                      ;; would cause a compiler error as it tries to decide
                      ;; whether any clause of this COND subsumes another.
                      ;; Moreover, we don't require the host to support MOP.
+                     #-sb-xc-host
                      ((sb!pcl::classp x) (translate (sb!pcl::class-classoid x)))
                      #-sb-xc-host
                      ((sb!pcl::eql-specializer-p type-specifier)
@@ -961,7 +813,7 @@
                      (t (fail x))))))
     (when (typep type-specifier 'instance)
       (return-from values-specifier-type-r (instance-to-ctype type-specifier)))
-    (values-specifier-type-memo-wrapper
+    (!values-specifier-type-memo-wrapper
      (lambda ()
        (labels
          ((recurse (spec)
@@ -979,12 +831,13 @@
                 (case (info :type :kind spec)
                   (:instance (return (find-classoid spec)))
                   (:forthcoming-defclass-type (go unknown))))
-              (awhen (info :type :translator head)
-                (return (or (funcall it context spec) (fail spec))))
               ;; Expansion brings up an interesting question - should the cache
               ;; contain entries for intermediary types? Say A -> B -> REAL.
               ;; As it stands, we cache the ctype corresponding to A but not B.
               (awhen (info :type :expander head)
+                (when (listp it) ; The function translates directly to a CTYPE.
+                  (return (or (funcall (car it) context spec) (fail spec))))
+                ;; The function produces a type expression.
                 (let ((expansion (funcall it (ensure-list spec))))
                   (return (if (typep expansion 'instance)
                               (instance-to-ctype expansion)
@@ -1049,7 +902,7 @@ expansion happened."
                ;; a) From a user's point of view, CL types are opaque.
                ;;
                ;; b) so (EQUAL (TYPEXPAND 'STRING) (TYPEXPAND-ALL 'STRING))
-    (if (and expander (not (info :type :builtin atom)))
+    (if (and (functionp expander) (not (info :type :builtin atom)))
         (values (funcall expander (if (symbolp spec) (list spec) spec)) t)
         (values type-specifier nil))))
 
