@@ -13,14 +13,6 @@
 
 ;;;; DYNAMIC-USAGE and friends
 
-#!-sb-fluid
-(declaim (inline current-dynamic-space-start))
-#!+gencgc
-(defun current-dynamic-space-start () sb!vm:dynamic-space-start)
-#!-gencgc
-(defun current-dynamic-space-start ()
-  (extern-alien "current_dynamic_space" unsigned-long))
-
 #!-sb-fluid
 (declaim (inline dynamic-usage))
 #!+gencgc
@@ -133,7 +125,7 @@ and submit it as a patch."
 
 ;;;; GC hooks
 
-(defvar *after-gc-hooks* nil
+(!defvar *after-gc-hooks* nil
   #!+sb-doc
   "Called after each garbage collection, except for garbage collections
 triggered during thread exits. In a multithreaded environment these hooks may
@@ -175,12 +167,13 @@ designated file is opened before and after each collection, and generation
 statistics are appended to it."
     (let ((val (cast %gc-logfile c-string)))
       (when val
-        (native-pathname val))))
-  (declaim (inline dynamic-space-size))
-  (defun dynamic-space-size ()
-    #!+sb-doc
-    "Size of the dynamic space in bytes."
-    (extern-alien "dynamic_space_size" os-vm-size-t)))
+        (native-pathname val)))))
+
+(declaim (inline dynamic-space-size))
+(defun dynamic-space-size ()
+  #!+sb-doc
+  "Size of the dynamic space in bytes."
+  (extern-alien "dynamic_space_size" os-vm-size-t))
 
 ;;;; SUB-GC
 
@@ -215,7 +208,7 @@ statistics are appended to it."
 ;;; comparisons. Unlikely, but the cost of using a cons instead is too
 ;;; small to measure. -- JES, 2007-09-30
 (declaim (type cons *gc-epoch*))
-(defvar *gc-epoch* (cons nil nil))
+(!defvar *gc-epoch* '(nil . nil))
 
 (defun sub-gc (&key (gen 0))
   (cond (*gc-inhibit*
