@@ -42,14 +42,14 @@
                  (symbol fdefn-or-symbol)
                  (fdefn (fdefn-name fdefn-or-symbol)))))
 
-#!+(or arm x86-64)
+#!+(or arm arm64 x86-64)
 (deferr undefined-alien-fun-error (address)
   (error 'undefined-alien-function-error
          :name
          (and (integerp address)
               (sap-foreign-symbol (int-sap address)))))
 
-#!-(or arm x86-64)
+#!-(or arm arm64 x86-64)
 (defun undefined-alien-fun-error ()
   (error 'undefined-alien-function-error))
 
@@ -79,9 +79,30 @@
          "attempt to RETURN-FROM a block or GO to a tag that no longer exists"))
 
 (deferr unseen-throw-tag-error (tag)
-  (error 'simple-control-error
-         :format-control "attempt to THROW to a tag that does not exist: ~S"
-         :format-arguments (list tag)))
+  (let ((text "attempt to THROW to a tag that does not exist: ~S"))
+    #!+sb-fasteval
+    (when (listp tag)
+      (multiple-value-bind (name frame)
+          (sb!debug::find-interrupted-name-and-frame)
+        (let ((down (and (eq name 'sb!c::unwind) ; is this tautological ?
+                         (sb!di:frame-down frame))))
+          (when frame
+            ;; Is this really the canonical way to get a frame name?
+            (let ((prev-frame-name
+                   (sb!di:debug-fun-name (sb!di:frame-debug-fun down))))
+              (when (and (listp prev-frame-name)
+                         (eq (car prev-frame-name) 'sb!c::xep))
+                (setq prev-frame-name (second prev-frame-name)))
+              (cond ((equal prev-frame-name '(eval return-from))
+                     (setq text "attempt to RETURN-FROM an exited block: ~S"
+                           ;; block name was wrapped in a cons
+                           tag (car tag)))
+                    ((equal prev-frame-name '(eval go))
+                     ;; FIXME: can we reverse-engineer the tag name from
+                     ;; the object that was thrown, for a better diagnostic?
+                     (setq text "attempt to GO into an exited tagbody"))))))))
+    (error 'simple-control-error
+           :format-control text :format-arguments (list tag))))
 
 (deferr nil-fun-returned-error (function)
   (error 'simple-control-error
@@ -156,27 +177,12 @@
       ((or error sb!di:debug-condition) ()))))
 
 (defun find-interrupted-frame ()
-  (/show0 "entering FIND-INTERRUPTED-FRAME")
-  (unless *finding-frame*
+  (when (plusp *free-interrupt-context-index*)
     (handler-case
-        (let ((*finding-frame* t))
-          (/show0 "in ordinary case")
-          (do ((frame (sb!di:top-frame) (sb!di:frame-down frame)))
-              ((null frame)
-               (/show0 "null frame")
-               nil)
-            (/noshow0 "at head of DO loop")
-            (when (and (sb!di::compiled-frame-p frame)
-                       (sb!di::compiled-frame-escaped frame))
-              (sb!di:flush-frames-above frame)
-              (/show0 "returning from within DO loop")
-              (return frame))))
-      (error ()
-        (/show0 "trapped ERROR")
-        nil)
-      (sb!di:debug-condition ()
-        (/show0 "trapped DEBUG-CONDITION")
-        nil))))
+        (sb!di::signal-context-frame (sb!alien::alien-sap
+                                      (sb!di::nth-interrupt-context
+                                       (1- *free-interrupt-context-index*))))
+      ((or error sb!di:debug-condition) ()))))
 
 (defun find-caller-of-named-frame (name)
   (unless *finding-frame*
@@ -345,8 +351,9 @@
 
 #!-win32
 (defun memory-fault-error ()
-  (error 'memory-fault-error
-         :address current-memory-fault-address))
+  (let ((sb!debug:*stack-top-hint* (find-interrupted-frame)))
+    (error 'memory-fault-error
+           :address current-memory-fault-address)))
 
 ;;; This is SIGTRAP / EXCEPTION_BREAKPOINT that runtime could not deal
 ;;; with. Prior to Windows we just had a Lisp side handler for
