@@ -57,7 +57,7 @@
          (slotd (or (and dd (find slot (dd-slots dd) :key #'dsd-name))
                     (error "Slot ~S not found in ~S." slot structure)))
          (index (dsd-index slotd))
-         #!-interleaved-raw-slots (raw-type (dsd-raw-type slotd)))
+         #!-interleaved-raw-slots (rsd (sb!kernel::dsd-raw-slot-data slotd)))
     `(progn
        (declaim (inline ,name))
        (defun ,name (instance)
@@ -69,11 +69,11 @@
                  #!+interleaved-raw-slots
                  (* (+ sb!vm:instance-slots-offset index) sb!vm:n-word-bytes)
                  #!-interleaved-raw-slots
-                 (* (if (eq t raw-type)
+                 (* (if (not rsd)
                         (+ sb!vm:instance-slots-offset index)
                         (- (1+ (sb!kernel::dd-instance-length dd))
                            sb!vm:instance-slots-offset index
-                           (1- (sb!kernel::raw-slot-words raw-type))))
+                           (1- (sb!kernel::raw-slot-data-n-words rsd))))
                     sb!vm:n-word-bytes))))))))
 
 ;;;; ATOMIC-INCF and ATOMIC-DECF
@@ -82,7 +82,16 @@
                            &aux (place (sb!xc:macroexpand specified-place env)))
   (declare (type (member atomic-incf atomic-decf) name))
   (flet ((invalid-place ()
-           (error "Invalid first argument to ~S: ~S" name specified-place)))
+           (error "Invalid first argument to ~S: ~S" name specified-place))
+         (compute-newval (old) ; used only if no atomic inc vop
+           `(logand (,(case name (atomic-incf '+) (atomic-decf '-)) ,old
+                     (the sb!vm:signed-word ,diff)) sb!ext:most-positive-word))
+         (compute-delta () ; used only with atomic inc vop
+           `(logand ,(case name
+                      (atomic-incf `(the sb!vm:signed-word ,diff))
+                      (atomic-decf `(- (the sb!vm:signed-word ,diff))))
+                    sb!ext:most-positive-word)))
+    (declare (ignorable #'compute-newval #'compute-delta))
     (if (and (symbolp place)
              (eq (info :variable :kind place) :global)
              (type= (info :variable :type place) (specifier-type 'fixnum)))
@@ -100,34 +109,22 @@
       ;; CL builtins can't be lexically rebound, but structure accessors can.
       (case op
         (aref
-         (when (cddr args)
+         (unless (singleton-p (cdr args))
            (invalid-place))
-         #!+(or x86 x86-64 ppc)
          (with-unique-names (array)
            `(let ((,array (the (simple-array word (*)) ,(car args))))
+              #!+compare-and-swap-vops
               (%array-atomic-incf/word
                ,array
-               (%check-bound ,array (array-dimension ,array 0) ,(cadr args))
-               (logand #.(1- (ash 1 sb!vm:n-word-bits))
-                       ,(case name
-                         (atomic-incf `(the sb!vm:signed-word ,diff))
-                         (atomic-decf `(- (the sb!vm:signed-word ,diff))))))))
-         #!-(or x86 x86-64 ppc)
-         (with-unique-names (array index old-value)
-           (let ((incremented-value
-                  (case name
-                   (atomic-incf
-                    `(+ ,old-value (the sb!vm:signed-word ,diff)))
-                   (atomic-decf
-                    `(- ,old-value (the sb!vm:signed-word ,diff))))))
-             `(without-interrupts
-                (let* ((,array ,(car args))
-                       (,index ,(cadr args))
-                       (,old-value (aref ,array ,index)))
-                  (setf (aref ,array ,index)
-                        (logand #.(1- (ash 1 sb!vm:n-word-bits))
-                                ,incremented-value))
-                  ,old-value)))))
+               (check-bound ,array (array-dimension ,array 0) ,(cadr args))
+               ,(compute-delta))
+              #!-compare-and-swap-vops
+              ,(with-unique-names (index old-value)
+                `(without-interrupts
+                  (let* ((,index ,(cadr args))
+                         (,old-value (aref ,array ,index)))
+                    (setf (aref ,array ,index) ,(compute-newval old-value))
+                    ,old-value))))))
         ((car cdr first rest)
          (when (cdr args)
            (invalid-place))
@@ -142,51 +139,36 @@
                             (atomic-decf '%atomic-dec-cdr))))
              ,(car args) (the fixnum ,diff))))
         (t
-         (when (cdr args)
-           (invalid-place))
-         ;; Without the SYMBOLP check this would erroneously allow
+         (when (or (cdr args)
+         ;; Because accessor info is identical for the writer and reader
+         ;; functions, without a SYMBOLP check this would erroneously allow
          ;;   (ATOMIC-INCF ((SETF STRUCT-SLOT) x))
-         (let ((info (and (symbolp op) (info :function :source-transform op))))
-           (if (consp info)
-               (let* ((dd (car info))
-                      (structure (dd-name dd))
-                      (slotd (cdr info))
-                      (index (dsd-index slotd))
-                      (type (dsd-type slotd)))
-                 (declare (ignorable structure index))
-                 (unless (and (eq 'sb!vm:word (dsd-raw-type slotd))
-                              (type= (specifier-type type) (specifier-type 'sb!vm:word)))
-                   (error "~S requires a slot of type (UNSIGNED-BYTE ~S), not ~S: ~S"
-                          name sb!vm:n-word-bits type place))
-                 (when (dsd-read-only slotd)
-                   (error "Cannot use ~S with structure accessor for a read-only slot: ~S"
-                          name place))
-                 #!+(or x86 x86-64 ppc)
-                 `(truly-the sb!vm:word
-                             (%raw-instance-atomic-incf/word
-                              (the ,structure ,@args) ,index
-                              (logand #.(1- (ash 1 sb!vm:n-word-bits))
-                                      ,(case name
-                                        (atomic-incf
-                                         `(the sb!vm:signed-word ,diff))
-                                        (atomic-decf
-                                         `(- (the sb!vm:signed-word ,diff)))))))
-                 ;; No threads outside x86 and x86-64 for now, so this is easy...
-                 #!-(or x86 x86-64 ppc)
-                 (with-unique-names (structure old)
-                   `(without-interrupts
-                      (let* ((,structure ,@args)
-                             (,old (,op ,structure)))
-                        (setf (,op ,structure)
-                              (logand
-                               #.(1- (ash 1 sb!vm:n-word-bits))
-                               ,(case name
-                                 (atomic-incf
-                                  `(+ ,old (the sb!vm:signed-word ,diff)))
-                                 (atomic-decf
-                                  `(- ,old (the sb!vm:signed-word ,diff))))))
-                        ,old))))
-             (invalid-place))))))))
+                   (not (symbolp op))
+                   (not (structure-instance-accessor-p op)))
+             (invalid-place))
+         (let* ((accessor-info (structure-instance-accessor-p op))
+                (slotd (cdr accessor-info))
+                (type (dsd-type slotd)))
+           (unless (and (eq 'sb!vm:word (dsd-raw-type slotd))
+                        (type= (specifier-type type) (specifier-type 'sb!vm:word)))
+             (error "~S requires a slot of type (UNSIGNED-BYTE ~S), not ~S: ~S"
+                    name sb!vm:n-word-bits type place))
+           (when (dsd-read-only slotd)
+             (error "Cannot use ~S with structure accessor for a read-only slot: ~S"
+                    name place))
+           #!+compare-and-swap-vops
+           `(truly-the sb!vm:word
+             (%raw-instance-atomic-incf/word
+              (the ,(dd-name (car accessor-info)) ,@args)
+              ,(dsd-index slotd)
+              ,(compute-delta)))
+           #!-compare-and-swap-vops
+           (with-unique-names (structure old-value)
+             `(without-interrupts
+               (let* ((,structure ,@args)
+                      (,old-value (,op ,structure)))
+                 (setf (,op ,structure) ,(compute-newval old-value))
+                 ,old-value)))))))))
 
 (def!macro atomic-incf (&environment env place &optional (diff 1))
   #!+sb-doc
@@ -249,7 +231,7 @@ EXPERIMENTAL: Interface subject to change."
   (expand-atomic-frob 'atomic-decf place diff env))
 
 ;; Interpreter stubs for ATOMIC-INCF.
-#!+(or x86 x86-64 ppc)
+#!+compare-and-swap-vops
 (progn
   ;; argument types are declared in vm-fndb
   (defun %array-atomic-incf/word (array index diff)
@@ -330,12 +312,13 @@ See also DEFGLOBAL which assigns the VALUE at compile-time too."
   (%compiler-defglobal name :always-bound value (not boundp))
   (when docp
     (setf (fdocumentation name 'variable) doc))
-  (sb!c:with-source-location (source-location)
+  (when source-location
     (setf (info :source-location :variable name) source-location))
   name)
 
 ;;;; WAIT-FOR -- waiting on arbitrary conditions
 
+#-sb-xc-host
 (defun %%wait-for (test stop-sec stop-usec)
   (declare (function test))
   (labels ((try ()
@@ -393,6 +376,7 @@ See also DEFGLOBAL which assigns the VALUE at compile-time too."
                        (t
                         (return-from %%wait-for nil))))))))
 
+#-sb-xc-host
 (defun %wait-for (test timeout)
   (declare (function test))
   (tagbody
@@ -448,8 +432,8 @@ returns NIL each time."
 designated by UPDATE-FN with ARGUMENTS and the previous value of PLACE.
 
 PLACE may be read and UPDATE-FN evaluated and called multiple times before the
-update succeeds: atomicity in this context means that value of place did not
-change between the time it was read, and the time it was replaced with the
+update succeeds: atomicity in this context means that the value of PLACE did
+not change between the time it was read, and the time it was replaced with the
 computed value.
 
 PLACE can be any place supported by SB-EXT:COMPARE-AND-SWAP.
@@ -553,3 +537,71 @@ version 1[.0.0...] or greater."
                or later is required)."
               (lisp-implementation-version)
               subversions))))
+
+;;; Signalling an error when trying to print an error condition is
+;;; generally a PITA, so whatever the failure encountered when
+;;; wondering about FILE-POSITION within a condition printer, 'tis
+;;; better silently to give up than to try to complain.
+(defun file-position-or-nil-for-error (stream &optional (pos nil posp))
+  ;; Arguably FILE-POSITION shouldn't be signalling errors at all; but
+  ;; "NIL if this cannot be determined" in the ANSI spec doesn't seem
+  ;; absolutely unambiguously to prohibit errors when, e.g., STREAM
+  ;; has been closed so that FILE-POSITION is a nonsense question. So
+  ;; my (WHN) impression is that the conservative approach is to
+  ;; IGNORE-ERRORS. (I encountered this failure from within a homebrew
+  ;; defsystemish operation where the ERROR-STREAM had been CL:CLOSEd,
+  ;; I think by nonlocally exiting through a WITH-OPEN-FILE, by the
+  ;; time an error was reported.)
+  (ignore-errors
+   (if posp
+       (file-position stream pos)
+       (file-position stream))))
+
+(defun stream-error-position-info (stream &optional position)
+  (when (and (not position) (form-tracking-stream-p stream))
+    (let ((line/col (line/col-from-charpos stream)))
+      (return-from stream-error-position-info
+        `((:line ,(car line/col))
+          (:column ,(cdr line/col))
+          ,@(let ((position (file-position-or-nil-for-error stream)))
+              ;; FIXME: 1- is technically broken for multi-byte external
+              ;; encodings, albeit bug-compatible with the broken code in
+              ;; the general case (below) for non-form-tracking-streams.
+              ;; i.e. If you position to this byte, it might not be the
+              ;; first byte of any character.
+              (when position `((:file-position ,(1- position)))))))))
+
+  ;; Give up early for interactive streams and non-character stream.
+  (when (or (ignore-errors (interactive-stream-p stream))
+            (not (subtypep (ignore-errors (stream-element-type stream))
+                           'character)))
+    (return-from stream-error-position-info))
+
+  (flet ((read-content (old-position position)
+           "Read the content of STREAM into a buffer in order to count
+lines and columns."
+           (unless (and old-position position
+                        (< position sb!xc:array-dimension-limit))
+             (return-from read-content))
+           (let ((content
+                   (make-string position :element-type (stream-element-type stream))))
+             (when (and (file-position-or-nil-for-error stream :start)
+                        (eql position (ignore-errors (read-sequence content stream))))
+               (file-position-or-nil-for-error stream old-position)
+               content)))
+         ;; Lines count from 1, columns from 0. It's stupid and
+         ;; traditional.
+         (line (string)
+           (1+ (count #\Newline string)))
+         (column (string position)
+           (- position (or (position #\Newline string :from-end t) 0))))
+   (let* ((stream-position (file-position-or-nil-for-error stream))
+          (position (or position
+                        ;; FILE-POSITION is the next character --
+                        ;; error is at the previous one.
+                        (and stream-position (plusp stream-position)
+                             (1- stream-position))))
+          (content (read-content stream-position position)))
+     `(,@(when content `((:line ,(line content))
+                         (:column ,(column content position))))
+       ,@(when position `((:file-position ,position)))))))
