@@ -18,7 +18,7 @@
 ;;;; -- WHN 20000127
 
 (declaim (maybe-inline
-          tree-equal nth %setnth nthcdr make-list
+          tree-equal %setnth nthcdr
           tailp union
           nunion intersection nintersection set-difference nset-difference
           set-exclusive-or nset-exclusive-or subsetp acons
@@ -334,6 +334,10 @@
 (defun make-list (size &key initial-element)
   #!+sb-doc
   "Constructs a list with size elements each set to value"
+  (%make-list size initial-element))
+;;; This entry point is to be preferred, irrespective of
+;;; whether or not the backend has vops for %MAKE-LIST.
+(defun %make-list (size initial-element)
   (declare (type index size))
   (do ((count size (1- count))
        (result '() (cons initial-element result)))
@@ -1365,6 +1369,7 @@
                      (declare (optimize speed (sb!c::verify-arg-count 0)))
                      ,@(when funs `((declare (function ,@funs))))
                      ,@(unless (member name '(member assoc adjoin rassoc)) `((declare (function x))))
+                     (declare (explicit-check))
                      ,body))))
          `(progn
             ,(%def 'adjoin)
