@@ -31,7 +31,7 @@
         (dx-let ((setter `(setf ,fun)))
           (when (and (not (sb!c::fun-locally-defined-p setter env))
                      (not (sb!c::fun-lexically-notinline-p setter env)))
-            slot-info)))))) ; caller needs the (DD . DSD) pair pack
+            slot-info)))))) ; caller needs the (DD . DSD) pair
 
 ;;; The inverse for a generalized-variable reference function is stored in
 ;;; one of two ways:
@@ -189,11 +189,11 @@
    returning the value of the leftmost."
   (when (< (length args) 2)
     (error "~S called with too few arguments: ~S" 'shiftf form))
-  (collect ((let-bindings) (mv-bindings) (setters) (getters))
+  (collect ((let*-bindings) (mv-bindings) (setters) (getters))
     (dolist (arg (butlast args))
       (multiple-value-bind (temps subforms store-vars setter getter)
           (sb!xc:get-setf-expansion arg env)
-        (let-bindings (mapcar #'list  temps subforms))
+        (let*-bindings (mapcar #'list  temps subforms))
         (mv-bindings store-vars)
         (setters setter)
         (getters getter)))
@@ -207,11 +207,11 @@
                    setters)))
       (let ((outputs (loop for i below (length (car (mv-bindings)))
                            collect (sb!xc:gensym "OUT"))))
-        `(let ,(reduce #'append (let-bindings))
-           ,@(gen-mv-bind outputs (car (getters))
-                          (thunk (mv-bindings) (cdr (getters))
-                                 `(,@(de-values-ify (setters))
-                                   (values ,@outputs)))))))))
+        (car (gen-let* (reduce #'nconc (let*-bindings))
+                       (gen-mv-bind outputs (car (getters))
+                                    (thunk (mv-bindings) (cdr (getters))
+                                           `(,@(de-values-ify (setters))
+                                             (values ,@outputs))))))))))
 
   (labels
       ((expand (args env operator single-op)
@@ -490,7 +490,7 @@
                                '(&optional &rest &key &allow-other-keys
                                  &environment))
                     :context "a DEFSETF lambda list"))
-                  ((forms decls doc) (parse-body body))
+                  ((forms decls doc) (parse-body body t))
                   ((outer-decls inner-decls)
                    (extract-var-decls decls (append env stores)))
                   (subforms (copy-symbol 'subforms))
