@@ -14,8 +14,7 @@
 ;;; We compile some trivial character operations via inline expansion.
 #!-sb-fluid
 (declaim (inline standard-char-p graphic-char-p alpha-char-p
-                 upper-case-p lower-case-p both-case-p alphanumericp
-                 char-int))
+                 upper-case-p lower-case-p both-case-p alphanumericp))
 (declaim (maybe-inline digit-char-p))
 
 (deftype char-code ()
@@ -139,6 +138,11 @@
                                               (dotimes (i len)
                                                 (push (read-codepoint) ret))
                                               (nreverse ret))))))
+                            ;; Dependency cycle: BOTH-CASE-INDEX-P is defined later
+                            ;; because it uses **CHARACTER-MISC-DATABASE** which is
+                            ;; defined by the DEFGLOBAL above. This declaration
+                            ;; suppresses the warning that it can't be inlined.
+                            (declare (notinline both-case-index-p))
                             (loop until (>= index length)
                                   for key = (read-codepoint)
                                   for upper = (read-length-tagged)
@@ -667,7 +671,7 @@ is either numeric or alphabetic."
            (= (equal-char-code c1) (equal-char-code c2)))))))
 
 (defun char-equal-constant (x char reverse-case-char)
-  (declare (type character x))
+  (declare (type character x) (explicit-check))
   (or (eq char x)
       (eq reverse-case-char x)))
 
@@ -705,10 +709,11 @@ Case is ignored."))
 (defun two-arg-char-not-lessp (c1 c2)
   (>= (equal-char-code c1) (equal-char-code c2)))
 
-(macrolet ((def (op test doc)
+(macrolet ((def (op test doc &optional explicit-check)
              (declare (ignorable doc))
              `(defun ,op (character &rest more-characters)
                 #!+sb-doc ,doc
+                ,@(when explicit-check `((declare (explicit-check))))
                 (let ((c1 character))
                   (declare (character c1))
                   (do-rest-arg ((c2 i) more-characters 0 t)
@@ -731,19 +736,19 @@ Case is ignored."))
   ;; case-insensitive
   (def char-equal (two-arg-char-equal c1 c2)
     "Return T if all of the arguments are the same character.
-Case is ignored.")
+Case is ignored." t)
   (def char-lessp (two-arg-char-lessp c1 c2)
     "Return T if the arguments are in strictly increasing alphabetic order.
-Case is ignored.")
+Case is ignored." t)
   (def char-greaterp (two-arg-char-greaterp c1 c2)
     "Return T if the arguments are in strictly decreasing alphabetic order.
-Case is ignored.")
+Case is ignored." t)
   (def char-not-greaterp (two-arg-char-not-greaterp c1 c2)
     "Return T if the arguments are in strictly non-decreasing alphabetic order.
-Case is ignored.")
+Case is ignored." t)
   (def char-not-lessp (two-arg-char-not-lessp c1 c2)
     "Return T if the arguments are in strictly non-increasing alphabetic order.
-Case is ignored."))
+Case is ignored." t))
 
 
 ;;;; miscellaneous functions
@@ -794,3 +799,32 @@ character exists."
   (and (typep weight 'fixnum)
        (>= weight 0) (< weight radix) (< weight 36)
        (code-char (if (< weight 10) (+ 48 weight) (+ 55 weight)))))
+
+;;; Moved from 'string' because ALPHANUMERICP wants to be inlined,
+;;; and moving ALPHANUMERICP earlier causes a snowball effect of
+;;; other inlining failures.
+(flet ((%capitalize (string start end)
+         (declare (string string) (index start) (type sequence-end end))
+         (let ((saved-header string))
+           (with-one-string (string start end)
+             (do ((index start (1+ index))
+                  (new-word? t)
+                  (char nil))
+                 ((= index (the fixnum end)))
+               (declare (fixnum index))
+               (setq char (schar string index))
+               (cond ((not (alphanumericp char))
+                      (setq new-word? t))
+                     (new-word?
+                      ;; CHAR is the first case-modifiable character after
+                      ;; a sequence of non-case-modifiable characters.
+                      (setf (schar string index) (char-upcase char))
+                      (setq new-word? nil))
+                     (t
+                      (setf (schar string index) (char-downcase char))))))
+           saved-header)))
+(defun string-capitalize (string &key (start 0) end)
+  (%capitalize (copy-seq (string string)) start end))
+(defun nstring-capitalize (string &key (start 0) end)
+  (%capitalize string start end))
+) ; FLET
