@@ -130,6 +130,10 @@ offending thread using THREAD-ERROR-THREAD."))
                (:timeout
                 (format s "Joining thread timed out: thread ~A ~
                            did not exit in time."
+                        (thread-error-thread c)))
+               (:self-join
+                (format s "In thread ~A, attempt to join the current ~
+                           thread."
                         (thread-error-thread c))))))
   #!+sb-doc
   (:documentation
@@ -326,7 +330,7 @@ terminating the main thread would terminate the entire process. If
 ALLOW-EXIT is true, aborting the main thread is equivalent to calling
 SB-EXT:EXIT code 1 and :ABORT NIL.
 
-Invoking the initial ABORT restart estabilished by MAKE-THREAD is
+Invoking the initial ABORT restart established by MAKE-THREAD is
 equivalent to calling ABORT-THREAD in other than main threads.
 However, whereas ABORT restart may be rebound, ABORT-THREAD always
 unwinds the entire thread. (Behaviour of the initial ABORT restart for
@@ -476,6 +480,28 @@ HOLDING-MUTEX-P."
 
 (sb!ext:defglobal **deadlock-lock** nil)
 
+#!+(or (not sb-thread) sb-futex)
+(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
+  #!+sb-doc
+  "Waitqueue type."
+  (name nil :type (or null thread-name))
+  #!+(and sb-thread sb-futex)
+  (token nil))
+
+#!+(and sb-thread (not sb-futex))
+(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
+  #!+sb-doc
+  "Waitqueue type."
+  (name nil :type (or null thread-name))
+  ;; For WITH-CAS-LOCK: because CONDITION-WAIT must be able to call
+  ;; %WAITQUEUE-WAKEUP without re-aquiring the mutex, we need a separate
+  ;; lock. In most cases this should be uncontested thanks to the mutex --
+  ;; the only case where that might not be true is when CONDITION-WAIT
+  ;; unwinds and %WAITQUEUE-DROP is called.
+  %owner
+  %head
+  %tail)
+
 ;;; Signals an error if owner of LOCK is waiting on a lock whose release
 ;;; depends on the current thread. Does not detect deadlocks from sempahores.
 (defun check-deadlock ()
@@ -744,32 +770,8 @@ IF-NOT-OWNER is :FORCE)."
 
 ;;;; Waitqueues/condition variables
 
-;; For possible DX-allocating, see the notinline counterpart after the
-;; PROGN below
-(declaim (inline make-waitqueue))
-#!+(or (not sb-thread) sb-futex)
-(defstruct (waitqueue (:constructor make-waitqueue (&key name)))
-  #!+sb-doc
-  "Waitqueue type."
-  (name nil :type (or null thread-name))
-  #!+(and sb-thread sb-futex)
-  (token nil))
-
 #!+(and sb-thread (not sb-futex))
 (progn
-  (defstruct (waitqueue (:constructor make-waitqueue (&key name)))
-    #!+sb-doc
-    "Waitqueue type."
-    (name nil :type (or null thread-name))
-    ;; For WITH-CAS-LOCK: because CONDITION-WAIT must be able to call
-    ;; %WAITQUEUE-WAKEUP without re-aquiring the mutex, we need a separate
-    ;; lock. In most cases this should be uncontested thanks to the mutex --
-    ;; the only case where that might not be true is when CONDITION-WAIT
-    ;; unwinds and %WAITQUEUE-DROP is called.
-    %owner
-    %head
-    %tail)
-
   (defun %waitqueue-enqueue (thread queue)
     (setf (thread-waiting-for thread) queue)
     (let ((head (waitqueue-%head queue))
@@ -814,7 +816,6 @@ IF-NOT-OWNER is :FORCE)."
                               (thread-waiting-for next) queue nil))
                (decf n)))
     nil))
-(declaim (notinline make-waitqueue))
 
 (def!method print-object ((waitqueue waitqueue) stream)
   (print-unreadable-object (waitqueue stream :type t :identity t)
@@ -831,6 +832,116 @@ IF-NOT-OWNER is :FORCE)."
     :structure waitqueue
     :slot token)
 
+(declaim (inline %condition-wait))
+(defun %condition-wait (queue mutex
+                        timeout to-sec to-usec stop-sec stop-usec deadlinep)
+  #!-sb-thread
+  (declare (ignore queue mutex to-sec to-usec stop-sec stop-usec deadlinep))
+  #!-sb-thread
+  (sb!ext:wait-for nil :timeout timeout) ; Yeah...
+  #!+sb-thread
+  (let ((me *current-thread*))
+    (barrier (:read))
+    (assert (eq me (mutex-%owner mutex)))
+    (let ((status :interrupted))
+      ;; Need to disable interrupts so that we don't miss grabbing
+      ;; the mutex on our way out.
+      (without-interrupts
+        (unwind-protect
+             (progn
+               #!-sb-futex
+               (progn
+                 (%with-cas-lock ((waitqueue-%owner queue))
+                   (%waitqueue-enqueue me queue))
+                 (release-mutex mutex)
+                 (setf status
+                       (or (flet ((wakeup ()
+                                    (barrier (:read))
+                                    (unless (eq queue (thread-waiting-for me))
+                                      :ok)))
+                             (declare (dynamic-extent #'wakeup))
+                             (allow-with-interrupts
+                               (sb!impl::%%wait-for #'wakeup stop-sec stop-usec)))
+                           :timeout)))
+               #!+sb-futex
+               (with-pinned-objects (queue me)
+                 (setf (waitqueue-token queue) me)
+                 (release-mutex mutex)
+                 ;; Now we go to sleep using futex-wait. If anyone else
+                 ;; manages to grab MUTEX and call CONDITION-NOTIFY during
+                 ;; this comment, it will change the token, and so futex-wait
+                 ;; returns immediately instead of sleeping. Ergo, no lost
+                 ;; wakeup. We may get spurious wakeups, but that's ok.
+                 (setf status
+                       (case (allow-with-interrupts
+                               (futex-wait (waitqueue-token-address queue)
+                                           (get-lisp-obj-address me)
+                                           ;; our way of saying "no
+                                           ;; timeout":
+                                           (or to-sec -1)
+                                           (or to-usec 0)))
+                         ((1)
+                          ;;  1 = ETIMEDOUT
+                          :timeout)
+                         (t
+                          ;; -1 = EWOULDBLOCK, possibly spurious wakeup
+                          ;;  0 = normal wakeup
+                          ;;  2 = EINTR, a spurious wakeup
+                          :ok)))))
+          #!-sb-futex
+          (%with-cas-lock ((waitqueue-%owner queue))
+            (if (eq queue (thread-waiting-for me))
+                (%waitqueue-drop me queue)
+                (unless (eq :ok status)
+                  ;; CONDITION-NOTIFY thinks we've been woken up, but really
+                  ;; we're unwinding. Wake someone else up.
+                  (%waitqueue-wakeup queue 1))))
+          ;; Update timeout for mutex re-aquisition unless we are
+          ;; already past the requested timeout.
+          (when (and (eq :ok status) to-sec)
+            (setf (values to-sec to-usec)
+                  (sb!impl::relative-decoded-times stop-sec stop-usec))
+            (when (and (zerop to-sec) (not (plusp to-usec)))
+              (setf status :timeout)))
+          ;; If we ran into deadline, try to get the mutex before
+          ;; signaling. If we don't unwind it will look like a normal
+          ;; return from user perspective.
+          (when (and (eq :timeout status) deadlinep)
+            (let ((got-it (%try-mutex mutex me)))
+              (allow-with-interrupts
+                (signal-deadline)
+                (cond (got-it
+                       (return-from %condition-wait t))
+                      (t
+                       ;; The deadline may have changed.
+                       (setf (values to-sec to-usec stop-sec stop-usec deadlinep)
+                             (decode-timeout timeout))
+                       (setf status :ok))))))
+          ;; Re-acquire the mutex for normal return.
+          (when (eq :ok status)
+            (unless (or (%try-mutex mutex me)
+                        (allow-with-interrupts
+                          (%wait-for-mutex mutex me timeout
+                                           to-sec to-usec
+                                           stop-sec stop-usec deadlinep)))
+              (setf status :timeout)))))
+      ;; Determine actual return value. :ok means (potentially
+      ;; spurious) wakeup => T. :timeout => NIL.
+      (case status
+        (:ok
+         (if timeout
+             (multiple-value-bind (sec usec)
+                 (sb!impl::relative-decoded-times stop-sec stop-usec)
+               (values t sec usec))
+             t))
+        (:timeout
+         nil)
+        (t
+         ;; The only case we return normally without re-acquiring
+         ;; the mutex is when there is a :TIMEOUT that runs out.
+         (bug "%CONDITION-WAIT: invalid status on normal return: ~S" status))))))
+(declaim (notinline %condition-wait))
+
 (defun condition-wait (queue mutex &key timeout)
   #!+sb-doc
   "Atomically release MUTEX and start waiting on QUEUE for till another thread
@@ -869,101 +980,12 @@ around the call, checking the the associated data:
       (push data *data*)
       (condition-notify *queue*)))
 "
-  #!-sb-thread
-  (declare (ignore queue))
   (assert mutex)
-  #!-sb-thread
-  (sb!ext:wait-for nil :timeout timeout) ; Yeah...
-  #!+sb-thread
-  (let ((me *current-thread*))
-    (barrier (:read))
-    (assert (eq me (mutex-%owner mutex)))
+  (locally (declare (inline %condition-wait))
     (multiple-value-bind (to-sec to-usec stop-sec stop-usec deadlinep)
         (decode-timeout timeout)
-      (let ((status :interrupted))
-        ;; Need to disable interrupts so that we don't miss grabbing the
-        ;; mutex on our way out.
-        (without-interrupts
-          (unwind-protect
-               (progn
-                 #!-sb-futex
-                 (progn
-                   (%with-cas-lock ((waitqueue-%owner queue))
-                     (%waitqueue-enqueue me queue))
-                   (release-mutex mutex)
-                   (setf status
-                         (or (flet ((wakeup ()
-                                      (barrier (:read))
-                                      (unless (eq queue (thread-waiting-for me))
-                                        :ok)))
-                               (declare (dynamic-extent #'wakeup))
-                               (allow-with-interrupts
-                                 (sb!impl::%%wait-for #'wakeup stop-sec stop-usec)))
-                             :timeout)))
-                 #!+sb-futex
-                 (with-pinned-objects (queue me)
-                   (setf (waitqueue-token queue) me)
-                   (release-mutex mutex)
-                   ;; Now we go to sleep using futex-wait. If anyone else
-                   ;; manages to grab MUTEX and call CONDITION-NOTIFY during
-                   ;; this comment, it will change the token, and so futex-wait
-                   ;; returns immediately instead of sleeping. Ergo, no lost
-                   ;; wakeup. We may get spurious wakeups, but that's ok.
-                   (setf status
-                         (case (allow-with-interrupts
-                                 (futex-wait (waitqueue-token-address queue)
-                                             (get-lisp-obj-address me)
-                                             ;; our way of saying "no
-                                             ;; timeout":
-                                             (or to-sec -1)
-                                             (or to-usec 0)))
-                           ((1)
-                            ;;  1 = ETIMEDOUT
-                            :timeout)
-                           (t
-                            ;; -1 = EWOULDBLOCK, possibly spurious wakeup
-                            ;;  0 = normal wakeup
-                            ;;  2 = EINTR, a spurious wakeup
-                            :ok)))))
-            #!-sb-futex
-            (%with-cas-lock ((waitqueue-%owner queue))
-              (if (eq queue (thread-waiting-for me))
-                  (%waitqueue-drop me queue)
-                  (unless (eq :ok status)
-                    ;; CONDITION-NOTIFY thinks we've been woken up, but really
-                    ;; we're unwinding. Wake someone else up.
-                    (%waitqueue-wakeup queue 1))))
-            ;; Update timeout for mutex re-aquisition.
-            (when (and (eq :ok status) to-sec)
-              (setf (values to-sec to-usec)
-                    (sb!impl::relative-decoded-times stop-sec stop-usec)))
-            ;; If we ran into deadline, try to get the mutex before
-            ;; signaling. If we don't unwind it will look like a normal
-            ;; return from user perspective.
-            (when (and (eq :timeout status) deadlinep)
-              (let ((got-it (%try-mutex mutex me)))
-                (allow-with-interrupts
-                  (signal-deadline)
-                  (cond (got-it
-                         (return-from condition-wait t))
-                        (t
-                         ;; The deadline may have changed.
-                         (setf (values to-sec to-usec stop-sec stop-usec deadlinep)
-                               (decode-timeout timeout))
-                         (setf status :ok))))))
-            ;; Re-acquire the mutex for normal return.
-            (when (eq :ok status)
-              (unless (or (%try-mutex mutex me)
-                          (allow-with-interrupts
-                            (%wait-for-mutex mutex me timeout
-                                             to-sec to-usec
-                                             stop-sec stop-usec deadlinep)))
-                (setf status :timeout)))))
-        (or (eq :ok status)
-            (unless (eq :timeout status)
-              ;; The only case we return normally without re-acquiring the
-              ;; mutex is when there is a :TIMEOUT that runs out.
-              (bug "CONDITION-WAIT: invalid status on normal return: ~S" status)))))))
+      (%condition-wait queue mutex timeout
+                       to-sec to-usec stop-sec stop-usec deadlinep))))
 
 (defun condition-notify (queue &optional (n 1))
   #!+sb-doc
@@ -1012,7 +1034,6 @@ must be held by this thread during this call."
 
 ;;;; Semaphores
 
-(declaim (inline make-semaphore)) ;; for possible DX-allocating
 (defstruct (semaphore (:constructor make-semaphore
                           (&key name ((:count %count) 0))))
   #!+sb-doc
@@ -1024,7 +1045,6 @@ future."
   (waitcount 0 :type sb!vm:word)
   (mutex (make-mutex))
   (queue (make-waitqueue)))
-(declaim (notinline make-semaphore))
 
 #!+sb-doc
 (setf (fdocumentation 'semaphore-name 'function)
@@ -1105,8 +1125,11 @@ WAIT-ON-SEMAPHORE or TRY-SEMAPHORE."
          nil)
         (t
          (unwind-protect
-              (let (old-count
-                    (timeout (when (realp wait) wait)))
+              (binding* ((old-count nil)
+                         (timeout (when (realp wait) wait))
+                         ((to-sec to-usec stop-sec stop-usec deadlinep)
+                          (when wait
+                            (decode-timeout timeout))))
                 ;; Need to use ATOMIC-INCF despite the lock, because
                 ;; on our way out from here we might not be locked
                 ;; anymore -- so another thread might be tweaking this
@@ -1115,10 +1138,18 @@ WAIT-ON-SEMAPHORE or TRY-SEMAPHORE."
                 ;; thread waiting on the semaphore.
                 (sb!ext:atomic-incf (semaphore-waitcount semaphore))
                 (loop until (>= (setf old-count (semaphore-%count semaphore)) n)
-                   do (or (condition-wait (semaphore-queue semaphore)
-                                          (semaphore-mutex semaphore)
-                                          :timeout timeout)
-                          (return-from %decrement-semaphore nil)))
+                   do (multiple-value-bind (wakeup-p remaining-sec remaining-usec)
+                          (%condition-wait
+                           (semaphore-queue semaphore)
+                           (semaphore-mutex semaphore)
+                           timeout to-sec to-usec stop-sec stop-usec deadlinep)
+                        (when (or (not wakeup-p)
+                                  (and (eql remaining-sec 0)
+                                       (eql remaining-usec 0)))
+                          (return-from %decrement-semaphore nil)) ; timeout
+                        (when remaining-sec
+                          (setf to-sec remaining-sec
+                                to-usec remaining-usec))))
                 (success (- old-count n)))
            ;; Need to use ATOMIC-DECF as we may unwind without the
            ;; lock being held!
@@ -1427,6 +1458,7 @@ session."
          (sb!impl::*previous-case* nil)
          (sb!impl::*previous-readtable-case* nil)
          (sb!impl::*internal-symbol-output-fun* #'error)
+         (sb!impl::*ignored-package-locks* :invalid)
          (sb!impl::*descriptor-handlers* nil)) ; serve-event
     (declare (inline make-restart)) ;; to allow DX-allocation
     ;; Binding from C
@@ -1523,23 +1555,16 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
                      make-mutex))
     (let* ((setup-sem (make-semaphore :name "Thread setup semaphore"))
            (real-function (coerce function 'function))
-           (arguments     (if (listp arguments)
-                              arguments
-                              (list arguments)))
-           #!+win32
+           (arguments     (ensure-list arguments))
+           #!+(or win32 darwin)
            (fp-modes (dpb 0 sb!vm::float-sticky-bits ;; clear accrued bits
                           (sb!vm:floating-point-modes)))
            ;; Allocate in the parent
            (thread-list (list thread thread)))
       (declare (dynamic-extent setup-sem))
-      ;; It could be just pinned instead, but implementing DX closures
-      ;; is much easier than implementing threading.
-      #!-(or stack-allocatable-closures stack-allocatable-fixed-objects)
-      #.(error "Stack allocatable closures required.")
       (dx-flet ((initial-thread-function ()
-                  ;; Win32 doesn't inherit parent thread's FP modes,
-                  ;; while it seems to happen everywhere else
-                  #!+win32
+                  ;; Inherit parent thread's FP modes
+                  #!+(or win32 darwin)
                   (setf (sb!vm:floating-point-modes) fp-modes)
                   ;; As it is, this lambda must not cons until we are
                   ;; ready to run GC. Be very careful.
@@ -1565,22 +1590,28 @@ See also: RETURN-FROM-THREAD, ABORT-THREAD."
   "Suspend current thread until THREAD exits. Return the result values
 of the thread function.
 
-If the thread does not exit within TIMEOUT seconds and DEFAULT is
+If THREAD does not exit within TIMEOUT seconds and DEFAULT is
 supplied, return two values: 1) DEFAULT 2) :TIMEOUT. If DEFAULT is not
-supplied, signal a JOIN-THREAD-ERROR with JOIN-THREAD-ERROR-PROBLEM
-equal to :TIMEOUT.
+supplied, signal a JOIN-THREAD-ERROR with JOIN-THREAD-PROBLEM equal
+to :TIMEOUT.
 
-If the thread did not exit normally (i.e. aborted) and DEFAULT is
+If THREAD did not exit normally (i.e. aborted) and DEFAULT is
 supplied, return two values: 1) DEFAULT 2) :ABORT. If DEFAULT is not
-supplied, signal a JOIN-THREAD-THREAD-ERROR with problem equal
+supplied, signal a JOIN-THREAD-ERROR with JOIN-THREAD-PROBLEM equal
 to :ABORT.
 
+If THREAD is the current thread, signal a JOIN-THREAD-ERROR with
+JOIN-THREAD-PROBLEM equal to :SELF-JOIN.
+
 Trying to join the main thread will cause JOIN-THREAD to block until
 TIMEOUT occurs or the process exits: when main thread exits, the
 entire process exits.
 
 NOTE: Return convention in case of a timeout is experimental and
 subject to change."
+  (when (eq thread *current-thread*)
+    (error 'join-thread-error :thread thread :problem :self-join))
+
   (let ((lock (thread-result-lock thread))
         (got-it nil)
         (problem :timeout))
@@ -1605,10 +1636,13 @@ subject to change."
         (error 'join-thread-error :thread thread :problem problem))))
 
 (defun destroy-thread (thread)
-  #!+sb-doc
-  "Deprecated. Same as TERMINATE-THREAD."
   (terminate-thread thread))
 
+#-sb-xc-host
+(declaim (sb!ext:deprecated
+          :late ("SBCL" "1.2.15")
+          (function destroy-thread :replacement terminate-thread)))
+
 #!+sb-thread
 (defun enter-foreign-callback (arg1 arg2 arg3)
   (initial-thread-function-trampoline
@@ -1802,13 +1836,12 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
                 (sap-ref-sap thread-sap (* sb!vm:n-word-bytes
                                            sb!vm::thread-next-slot)))))))
 
-  (defun %symbol-value-in-thread (symbol thread)
+(defun %symbol-value-in-thread (symbol thread)
     ;; Prevent the thread from dying completely while we look for the TLS
     ;; area...
     (with-all-threads-lock
       (if (thread-alive-p thread)
-          (let* ((offset (get-lisp-obj-address
-                          (sb!vm::symbol-tls-index symbol)))
+          (let* ((offset (get-lisp-obj-address (symbol-tls-index symbol)))
                  (obj (sap-ref-lispobj (%thread-sap thread) offset))
                  (tl-val (get-lisp-obj-address obj)))
             (cond ((zerop offset)
@@ -1826,8 +1859,7 @@ assume that unknown code can safely be terminated using TERMINATE-THREAD."
       ;; area...
       (with-all-threads-lock
         (if (thread-alive-p thread)
-            (let ((offset (get-lisp-obj-address
-                           (sb!vm::symbol-tls-index symbol))))
+            (let ((offset (get-lisp-obj-address (symbol-tls-index symbol))))
               (cond ((zerop offset)
                      (values nil :no-tls-value))
                     (t
@@ -1922,11 +1954,10 @@ mechanism for inter-thread communication."
 ;;;; Stepping
 
 (defun thread-stepping ()
-  (make-lisp-obj
-   (sap-ref-word (current-thread-sap)
-                 (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes))))
+  (sap-ref-lispobj (current-thread-sap)
+                   (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes)))
 
 (defun (setf thread-stepping) (value)
-  (setf (sap-ref-word (current-thread-sap)
-                      (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes))
-        (get-lisp-obj-address value)))
+  (setf (sap-ref-lispobj (current-thread-sap)
+                         (* sb!vm::thread-stepping-slot sb!vm:n-word-bytes))
+        value))
