@@ -15,6 +15,16 @@
 ;;; but such nuance isn't hugely important.
 (in-package "SB!C")
 
+(declaim (ftype (function (t t t) (values t t &optional)) info)
+         (ftype (function (t t t) (values t &optional)) clear-info)
+         (ftype (function (t t t t) (values t &optional)) (setf info)))
+
+;;; (:FUNCTION :TYPE) information is extracted through a wrapper.
+;;; The globaldb representation is not necessarily literally a CTYPE.
+#-sb-xc-host
+(declaim (ftype (function (t) (values ctype boolean &optional))
+                proclaimed-ftype))
+
 ;;; At run time, we represent the type of a piece of INFO in the globaldb
 ;;; by a small integer between 1 and 63.  [0 is reserved for internal use.]
 ;;; CLISP, and maybe others, need EVAL-WHEN because without it, the constant
@@ -62,6 +72,12 @@
 ;; Refer to info-vector.lisp for the meaning of this constant.
 (defconstant +no-auxilliary-key+ 0)
 
+;;; SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
+;;; But in the host Lisp, there is no such thing as a symbol-info slot.
+;;; Instead, symbol-info is kept in the host symbol's plist.
+#+sb-xc-host
+(defmacro symbol-info-vector (symbol) `(get ,symbol :sb-xc-globaldb-info))
+
 ;; Perform the equivalent of (GET-INFO-VALUE KIND +INFO-METAINFO-TYPE-NUM+)
 ;; but skipping the defaulting logic.
 ;; Return zero or more META-INFOs that match on KIND, which is usually
@@ -75,6 +91,14 @@
                                               +info-metainfo-type-num+))))
      (if index (svref info-vector index))))
 
+;; (UNSIGNED-BYTE 16) is an arbitrarily generous limit on the number of
+;; cells in an info-vector. Most vectors have a fewer than a handful of things,
+;; and performance would need to be re-thought if more than about a dozen
+;; cells were in use. (It would want to become hash-based probably)
+(declaim (ftype (function (simple-vector (or (eql 0) symbol) info-number)
+                          (or null (unsigned-byte 16)))
+                packed-info-value-index))
+
 ;; Return the META-INFO object for CATEGORY and KIND, signaling an error
 ;; if not found and ERRORP is non-nil. Note that the two-level logical hierarchy
 ;; of CATEGORY + KIND is physically grouped by KIND first, then CATEGORY.
@@ -126,7 +150,7 @@
                           ,form
                           (progn
                             #-sb-xc-host
-                            (style-warn "(INFO ~S ~S) will fail at runtime."
+                            (style-warn "(~S ~S) is not a defined info type."
                                         category kind)
                             .whole.)))
                     .whole.))))
@@ -139,6 +163,7 @@
     (let* (#+sb-xc-host (sb!xc:*gensym-counter* sb!xc:*gensym-counter*)
            (tin (meta-info-number meta-info)) ; info-type id number
            (type-spec (meta-info-type-spec meta-info))
+           (new (make-symbol "NEW"))
            (check
             (when (meta-info-validate-function meta-info)
               ;; is (or ... null), but non-null at macroexpansion time
@@ -146,14 +171,32 @@
               `(truly-the function
                 (meta-info-validate-function
                  (truly-the meta-info (svref *info-types* ,tin)))))))
-      (with-unique-names (new)
-        `(let ((,new ,new-value))
-           ;; enforce type-correctness regardless of enclosing policy
-           (let ((,new (locally (declare (optimize (safety 3)))
-                         (the ,type-spec ,new))))
-             ,@(when check
-                 `((funcall ,check ,name ,new)))
-             (set-info-value ,name ,tin ,new))))))
+      `(let ((,new ,new-value))
+         ;; enforce type-correctness regardless of enclosing policy
+         (let ((,new (locally (declare (optimize (safety 3)))
+                       (the ,type-spec ,new))))
+           ,@(when check
+               `((funcall ,check ,name ,new)))
+           (set-info-value ,name ,tin ,new)))))
 
   (def clear-info (category kind name)
     `(clear-info-values ,name '(,(meta-info-number meta-info)))))
+
+;; Perform the approximate equivalent operations of retrieving
+;; (INFO :CATEGORY :KIND NAME), but if no info is found, invoke CREATION-FORM
+;; to produce an object that becomes the value for that piece of info, storing
+;; and returning it. The entire sequence behaves atomically but with a proviso:
+;; the creation form's result may be discarded, and another object returned
+;; instead (presumably) from another thread's execution of the creation form.
+;; If constructing the object has either non-trivial cost, or deleterious
+;; side-effects from making and discarding its result, do NOT use this macro.
+;; A mutex-guarded table would probably be more appropriate in such cases.
+;;
+(defmacro get-info-value-initializing (category kind name creation-form)
+  (let ((proc (make-symbol "THUNK")))
+    `(dx-flet ((,proc () ,creation-form))
+       (%get-info-value-initializing
+        ,(if (and (keywordp category) (keywordp kind))
+             (meta-info-number (meta-info category kind))
+             `(meta-info-number (meta-info ,category ,kind)))
+        ,name #',proc))))
