@@ -26,7 +26,8 @@
   ;; and C-STACK-IS-CONTROL-STACK (otherwise, the C stack is the
   ;; number stack, and we precisely-scavenge the control stack).
   #!-(and :gencgc :c-stack-is-control-stack)
-  (zerop (dd-raw-length (lvar-value defstruct-description)))
+  (every (lambda (x) (eq (dsd-raw-type x) t))
+         (dd-slots (lvar-value defstruct-description)))
   #!+(and :gencgc :c-stack-is-control-stack)
   t)
 
@@ -78,38 +79,32 @@
             (slot (cdr init)))
         (case kind
           (:slot
+           ;; FIXME: with #!+interleaved-raw-slots the only reason INIT-SLOT
+           ;; and its raw variants exist is to avoid an extra MOVE -
+           ;; setters are expected to return something, but INITers don't.
+           ;; It would probably produce better code by not assuming that
+           ;; setters return a value, because as things are, if you call
+           ;; 8 setters in a row, then you probably produce 7 extraneous moves,
+           ;; because not all of them can deliver a value to the final result.
            (let* ((raw-type (pop slot))
                   (arg (pop args))
                   (arg-tn (lvar-tn node block arg)))
              (macrolet
-                 ((make-case ()
+                 ((make-case (&optional rsd-list)
                     `(ecase raw-type
                        ((t)
                         (vop init-slot node block object arg-tn
                              name (+ sb!vm:instance-slots-offset slot) lowtag))
-                       ,@(mapcar
+                       ,@(map 'list
                           (lambda (rsd)
-                            (let ((specifier (sb!kernel::raw-slot-data-raw-type
-                                              rsd)))
-                              `(,specifier
-                                (let ((type (specifier-type ',specifier))
-                                      (arg-tn arg-tn))
-                                  (unless (csubtypep (lvar-type arg) type)
-                                    (let ((tmp (make-normal-tn
-                                                (primitive-type type))))
-                                      (emit-type-check node block arg-tn
-                                                       tmp type)
-                                      (setf arg-tn tmp)))
-                                  (vop ,(sb!kernel::raw-slot-data-init-vop rsd)
-                                       node block
-                                       object arg-tn
-                                       #!-interleaved-raw-slots instance-length
-                                       slot)))))
-                          #!+raw-instance-init-vops
-                          sb!kernel::*raw-slot-data-list*
-                          #!-raw-instance-init-vops
-                          nil))))
-               (make-case))))
+                            `(,(sb!kernel::raw-slot-data-raw-type rsd)
+                              (vop ,(sb!kernel::raw-slot-data-init-vop rsd)
+                                   node block object arg-tn
+                                   #!-interleaved-raw-slots instance-length
+                                   slot)))
+                          (symbol-value rsd-list)))))
+               (make-case #!+raw-instance-init-vops
+                          sb!kernel::*raw-slot-data*))))
           (:dd
            (vop init-slot node block object
                 (emit-constant (sb!kernel::dd-layout-or-lose slot))
@@ -210,7 +205,12 @@
                                    (push
                                     `(,(sb!vm:saetp-typecode s)
                                        (lambda (index tn)
-                                         #!+(or x86 x86-64)
+                                         #!+x86-64
+                                         (vop ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/"
+                                                            (sb!vm:saetp-primitive-type-name s)
+                                                            "-C")
+                                              node block result tn index 0 tn)
+                                         #!+x86
                                          (vop ,(symbolicate "DATA-VECTOR-SET-WITH-OFFSET/"
                                                             (sb!vm:saetp-primitive-type-name s))
                                               node block result index tn 0 tn)
@@ -224,7 +224,10 @@
                          ,@(nreverse clauses)))))
                (frob)))
            (tnify (index)
-             (emit-constant index)))
+             #!-x86-64
+             (emit-constant index)
+             #!+x86-64
+             index))
       (let ((setter (compute-setter))
             (length (length initial-contents)))
         (dotimes (i length)
@@ -275,17 +278,25 @@
 
   (defoptimizer (allocate-vector ltn-annotate) ((type length words) call ltn-policy)
     (declare (ignore type length words))
-    (let ((args (basic-combination-args call))
-          (template (template-or-lose (if (awhen (node-lvar call)
-                                            (lvar-dynamic-extent it))
-                                          'sb!vm::allocate-vector-on-stack
-                                          'sb!vm::allocate-vector-on-heap))))
+    (vectorish-ltn-annotate-helper call ltn-policy
+                                   'sb!vm::allocate-vector-on-stack
+                                   'sb!vm::allocate-vector-on-heap))
+
+  (defun vectorish-ltn-annotate-helper (call ltn-policy dx-template &optional not-dx-template)
+    (let* ((args (basic-combination-args call))
+           (template-name (if (awhen (node-lvar call)
+                                (lvar-dynamic-extent it))
+                              dx-template
+                              not-dx-template))
+           (template (and template-name
+                          (template-or-lose template-name))))
       (dolist (arg args)
         (setf (lvar-info arg)
               (make-ir2-lvar (primitive-type (lvar-type arg)))))
-      (unless (is-ok-template-use template call (ltn-policy-safe-p ltn-policy))
+      (unless (and template
+                   (is-ok-template-use template call (ltn-policy-safe-p ltn-policy)))
         (ltn-default-call call)
-        (return-from allocate-vector-ltn-annotate-optimizer (values)))
+        (return-from vectorish-ltn-annotate-helper (values)))
       (setf (basic-combination-info call) template)
       (setf (node-tail-p call) nil)
 
@@ -314,3 +325,25 @@
   (defoptimizer (%make-complex stack-allocate-result) ((&rest args) node dx)
     (declare (ignore args dx))
     t))
+
+;;; MAKE-LIST optimizations
+#!+x86-64
+(progn
+  (defoptimizer (%make-list stack-allocate-result) ((length element) node dx)
+    (declare (ignore element))
+    (or (eq dx :always-dynamic)
+        (zerop (policy node safety))
+        ;; At most one page (this is more paranoid than %listify-rest-args).
+        ;; Really what you want to do is decrement the stack pointer by one page
+        ;; at a time, filling in CDR pointers downward. Then this restriction
+        ;; could be removed, because allocation would never miss the guard page
+        ;; if it tries to consume too much stack space.
+        (values-subtypep (lvar-derived-type length)
+                         (load-time-value
+                          (specifier-type `(integer 0 ,(/ sb!vm::*backend-page-bytes*
+                                                          sb!vm:n-word-bytes 2)))))))
+  (defoptimizer (%make-list ltn-annotate) ((length element) call ltn-policy)
+    (declare (ignore length element))
+    (vectorish-ltn-annotate-helper call ltn-policy
+                                   'sb!vm::allocate-list-on-stack)))
+
