@@ -412,7 +412,7 @@
                      (when variant
                        (write-char #\- s)
                        (write-string (symbol-name variant) s)))
-                   (load-time-value (find-package "SB!KERNEL")))
+                   (load-time-value (find-package "SB!KERNEL") t))
       (bug "Unknown list item seek transform: name=~S, key-functions=~S variant=~S"
            function-name key-functions variant)))
 
@@ -624,7 +624,8 @@
            (let* ((n-bits (sb!vm:saetp-n-bits saetp))
                   (basher-name (format nil "UB~D-BASH-FILL" n-bits))
                   (basher (or (find-symbol basher-name
-                                           (load-time-value (find-package "SB!KERNEL")))
+                                           (load-time-value
+                                            (find-package "SB!KERNEL") t))
                               (abort-ir1-transform
                                "Unknown fill basher, please report to sbcl-devel: ~A"
                                basher-name)))
@@ -710,12 +711,12 @@
                   `(let* ((len (length seq))
                           (end (or end len))
                           (bound (1+ end)))
-                     ;; Minor abuse %CHECK-BOUND for bounds checking.
+                     ;; Minor abuse CHECK-BOUND for bounds checking.
                      ;; (- END START) may still end up negative, but
                      ;; the basher handle that.
                      (,basher ,bash-value seq
-                              (%check-bound seq bound start)
-                              (- (if end (%check-bound seq bound end) len)
+                              (check-bound seq bound start)
+                              (- (if end (check-bound seq bound end) len)
                                  start)))
                `(with-array-data ((data seq)
                                   (start start)
@@ -915,7 +916,7 @@
 ;;; performance of, the functions implementing string streams
 ;;; (e.g. SB!IMPL::STRING-OUCH).
 (eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-  (defun make-replace-transform (saetp sequence-type1 sequence-type2)
+  (defun !make-replace-transform (saetp sequence-type1 sequence-type2)
     `(deftransform replace ((seq1 seq2 &key (start1 0) (start2 0) end1 end2)
                             (,sequence-type1 ,sequence-type2 &rest t)
                             ,sequence-type1
@@ -965,11 +966,11 @@
        (loop for saetp across sb!vm:*specialized-array-element-type-properties*
              for sequence-type = `(simple-array ,(sb!vm:saetp-specifier saetp) (*))
              unless (= (sb!vm:saetp-typecode saetp) sb!vm::simple-array-nil-widetag)
-             collect (make-replace-transform saetp sequence-type sequence-type)
+             collect (!make-replace-transform saetp sequence-type sequence-type)
              into forms
              finally (return `(progn ,@forms))))
      (define-one-transform (sequence-type1 sequence-type2)
-       (make-replace-transform nil sequence-type1 sequence-type2)))
+       (!make-replace-transform nil sequence-type1 sequence-type2)))
   (define-replace-transforms)
   #!+sb-unicode
   (progn
@@ -1122,75 +1123,79 @@
              (values spec (ir1-transform-specifier-type spec))))
     (unless type
       (give-up-ir1-transform))
-    (multiple-value-bind (elt-type dim complexp)
-        (cond ((and (union-type-p type)
-                    (csubtypep type (specifier-type 'string)))
-               (let* ((types (union-type-types type))
-                      (first (first types)))
-                 (when (array-type-p first)
-                   (let ((dim (first (array-type-dimensions first)))
-                         (complexp (array-type-complexp first)))
-                     ;; Require sameness of dim and complexp. Give up on
-                     ;;   (OR (VECTOR CHARACTER) (VECTOR BASE-CHAR 2))
-                     ;; which eventually fails in the call to the function.
-                     (when (every (lambda (x)
-                                    (and (array-type-p x)
-                                         (eql (first (array-type-dimensions x))
-                                              dim)
-                                         (eq (array-type-complexp x) complexp)))
-                                  (rest types))
-                       (values
-                        `(or ,@(mapcar
-                                (lambda (x)
-                                  (type-specifier (array-type-element-type x)))
-                                types))
-                        dim complexp))))))
-              ((and (array-type-p type)
-                    (csubtypep type (specifier-type 'vector)))
-               (when (contains-unknown-type-p (array-type-element-type type))
-                 (give-up-ir1-transform "~S is an unknown vector type" spec))
-               (values (let ((et (array-type-element-type type)))
-                         ;; VECTOR means (VECTOR T)
-                         (if (type= et *wild-type*) 't (type-specifier et)))
-                       (first (array-type-dimensions type))
-                       (array-type-complexp type))))
-      ;; Don't transform if size is present in the specifier
-      ;; and the SIZE argument is not known to be equal.
-      (if (and (or (eq '* dim)
-                   (and dim (constant-lvar-p size) (eql (lvar-value size) dim)))
-               ;; not sure what it would mean to make it non-simple
-               (neq complexp t))
-          `(make-array size :element-type ',elt-type
-                       ,@(when initial-element
-                           `(:initial-element initial-element)))
-          ;; no transform, but we can detect some style issues
-          (progn
-            (when dim ; was a recognizable vector subtype
-              (let* ((elt-ctype (specifier-type elt-type))
-                     (saetp (find-saetp-by-ctype elt-ctype)))
-                (cond ((not initial-element)
-                       (let ((default-initial-element
-                               (sb!vm:saetp-initial-element-default saetp)))
-                         (unless (ctypep default-initial-element elt-ctype)
-                           ;; As with MAKE-ARRAY, this is merely undefined
-                           ;; behavior, not an error.
-                           (compiler-style-warn
-                            "The default initial element ~S is not a ~S."
-                            default-initial-element elt-type))))
-                      ;; In would be possible in some cases,
-                      ;; like :INITIAL-ELEMENT (IF X #\x #\y) in a call
-                      ;; to MAKE-SEQUENCE '(VECTOR (MEMBER #\A #\B))
-                      ;; to detect erroneous non-constants initializers,
-                      ;; but it is not important enough to bother with.
-                      ((and (constant-lvar-p initial-element)
-                            (not (ctypep (lvar-value initial-element)
-                                         elt-ctype)))
-                       ;; MAKE-ARRAY considers this a warning, not an error.
-                       (compiler-warn "~S ~S is not a ~S"
-                                      :initial-element
-                                      (lvar-value initial-element)
-                                      elt-type)))))
-            (give-up-ir1-transform))))))
+    (if (type= type (specifier-type 'list))
+        `(%make-list size initial-element)
+        (multiple-value-bind (elt-type dim complexp)
+            (cond ((and (union-type-p type)
+                        (csubtypep type (specifier-type 'string)))
+                   (let* ((types (union-type-types type))
+                          (first (first types)))
+                     (when (array-type-p first)
+                       (let ((dim (first (array-type-dimensions first)))
+                             (complexp (array-type-complexp first)))
+                         ;; Require sameness of dim and complexp. Give up on
+                         ;;   (OR (VECTOR CHARACTER) (VECTOR BASE-CHAR 2))
+                         ;; which eventually fails in the call to the function.
+                         (when (every (lambda (x)
+                                        (and (array-type-p x)
+                                             (eql (first (array-type-dimensions x))
+                                                  dim)
+                                             (eq (array-type-complexp x) complexp)))
+                                      (rest types))
+                           (values
+                            `(or ,@(mapcar
+                                    (lambda (x)
+                                      (type-specifier (array-type-element-type x)))
+                                    types))
+                            dim complexp))))))
+                  ((and (array-type-p type)
+                        (csubtypep type (specifier-type 'vector)))
+                   (when (contains-unknown-type-p (array-type-element-type type))
+                     (give-up-ir1-transform "~S is an unknown vector type" spec))
+                   (values (let ((et (array-type-element-type type)))
+                             ;; VECTOR means (VECTOR T)
+                             (if (type= et *wild-type*)
+                                 t
+                                 (type-specifier et)))
+                           (first (array-type-dimensions type))
+                           (array-type-complexp type))))
+          ;; Don't transform if size is present in the specifier
+          ;; and the SIZE argument is not known to be equal.
+          (cond ((and (or (eq '* dim)
+                          (and dim (constant-lvar-p size) (eql (lvar-value size) dim)))
+                      ;; not sure what it would mean to make it non-simple
+                      (neq complexp t))
+                 `(make-array size :element-type ',elt-type
+                              ,@(when initial-element
+                                  `(:initial-element initial-element))))
+                ;; no transform, but we can detect some style issues
+                (t
+                 (when dim         ; was a recognizable vector subtype
+                   (let* ((elt-ctype (specifier-type elt-type))
+                          (saetp (find-saetp-by-ctype elt-ctype)))
+                     (cond ((not initial-element)
+                            (let ((default-initial-element
+                                    (sb!vm:saetp-initial-element-default saetp)))
+                              (unless (ctypep default-initial-element elt-ctype)
+                                ;; As with MAKE-ARRAY, this is merely undefined
+                                ;; behavior, not an error.
+                                (compiler-style-warn
+                                 "The default initial element ~S is not a ~S."
+                                 default-initial-element elt-type))))
+                           ;; In would be possible in some cases,
+                           ;; like :INITIAL-ELEMENT (IF X #\x #\y) in a call
+                           ;; to MAKE-SEQUENCE '(VECTOR (MEMBER #\A #\B))
+                           ;; to detect erroneous non-constants initializers,
+                           ;; but it is not important enough to bother with.
+                           ((and (constant-lvar-p initial-element)
+                                 (not (ctypep (lvar-value initial-element)
+                                              elt-ctype)))
+                            ;; MAKE-ARRAY considers this a warning, not an error.
+                            (compiler-warn "~S ~S is not a ~S"
+                                           :initial-element
+                                           (lvar-value initial-element)
+                                           elt-type)))))
+                 (give-up-ir1-transform)))))))
 
 (deftransform subseq ((seq start &optional end)
                       (vector t &optional t)
