@@ -560,12 +560,31 @@
                               (+ (dstate-cur-offs dstate)
                                  (inst-length inst)))
                         (let ((orig-next (dstate-next-offs dstate))
-                              (prefilter (inst-prefilter inst))
                               (control (inst-control inst)))
                           (print-inst (inst-length inst) stream dstate
                                       :trailing-space nil)
-                          (when prefilter
-                            (funcall prefilter chunk dstate))
+
+                          (dolist (item (inst-prefilters inst))
+                            (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+                            ;; item = #(INDEX FUNCTION SIGN-EXTEND-P BYTE-SPEC ...).
+                            (flet ((extract-byte (spec-index)
+                                     (let* ((byte-spec (svref item spec-index))
+                                            (integer (dchunk-extract chunk byte-spec)))
+                                       (if (svref item 2) ; SIGN-EXTEND-P
+                                           (sign-extend integer (byte-size byte-spec))
+                                           integer))))
+                              (let ((item-length (length item))
+                                    (fun (svref item 1)))
+                                (setf (svref (dstate-filtered-values dstate) (svref item 0))
+                                      (case item-length
+                                        (2 (funcall fun dstate)) ; no subfields
+                                        (3 (bug "Bogus prefilter"))
+                                        (4 (funcall fun dstate (extract-byte 3))) ; one subfield
+                                        (5 (funcall fun dstate ; two subfields
+                                                    (extract-byte 3) (extract-byte 4)))
+                                        (t (apply fun dstate ; > 2 subfields
+                                                  (loop for i from 3 below item-length
+                                                        collect (extract-byte i)))))))))
 
                           (setf prefix-p (null (inst-printer inst)))
 
@@ -612,6 +631,30 @@
         (setf (dstate-inst-properties dstate) nil)))))
 
 
+(defun collect-labelish-operands (args cache)
+  (awhen (remove-if-not #'arg-use-label args)
+    (let* ((list (mapcar (lambda (arg &aux (fun (arg-use-label arg))
+                                           (prefilter (arg-prefilter arg))
+                                           (bytes (arg-fields arg)))
+                           ;; Require byte specs or a prefilter (or both).
+                           ;; Prefilter alone is ok - it can use READ-SUFFIX.
+                           ;; Additionally, you can't have :use-label T
+                           ;; if multiple fields exist with no prefilter.
+                           (aver (or prefilter
+                                     (if (eq fun t) (singleton-p bytes) bytes)))
+                           ;; If arg has a prefilter, just compute its index,
+                           ;; otherwise keep the byte specs for extraction.
+                           (coerce (cons (if (eq fun t) #'identity fun)
+                                         (if prefilter
+                                             (list (posq arg args))
+                                             (cons (arg-sign-extend-p arg) bytes)))
+                                   'vector))
+                         it))
+           (repr (if (cdr list) list (car list))) ; usually just 1 item
+           (table (assq :labeller cache)))
+      (or (find repr (cdr table) :test 'equalp)
+          (car (push repr (cdr table)))))))
+
 ;;; Make an initial non-printing disassembly pass through DSTATE,
 ;;; noting any addresses that are referenced by instructions in this
 ;;; segment.
@@ -623,9 +666,39 @@
     (map-segment-instructions
      (lambda (chunk inst)
        (declare (type dchunk chunk) (type instruction inst))
-       (let ((labeller (inst-labeller inst)))
-         (when labeller
-           (setf labels (funcall labeller chunk labels dstate)))))
+       (declare (optimize (sb!c::insert-array-bounds-checks 0)))
+       (loop with list = (inst-labeller inst)
+             while list
+             ;; item = #(FUNCTION PREFILTERED-VALUE-INDEX)
+             ;;      | #(FUNCTION SIGN-EXTEND-P BYTE-SPEC ...)
+             for item = (if (listp list) (pop list) (prog1 list (setq list nil)))
+             then (pop list)
+          do (let* ((item-length (length item))
+                    (index/signedp (svref item 1))
+                    (adjusted-value
+                     (funcall
+                      (svref item 0)
+                      (flet ((extract-byte (spec-index)
+                               (let* ((byte-spec (svref item spec-index))
+                                      (integer (dchunk-extract chunk byte-spec)))
+                                 (if index/signedp
+                                     (sign-extend integer (byte-size byte-spec))
+                                     integer))))
+                        (case item-length
+                          (2 (svref (dstate-filtered-values dstate) index/signedp))
+                          (3 (extract-byte 2)) ; extract exactly one byte
+                          (t ; extract >1 byte.
+                           ;; FIXME: this is strictly redundant.
+                           ;; You should combine fields in the prefilter
+                           ;; so that the labeller receives a single byte.
+                           ;; AARCH64 and HPPA make use of this though.
+                           (loop for i from 2 below item-length
+                                 collect (extract-byte i)))))
+                      dstate)))
+               ;; If non-integer, the value is not a label.
+               (when (and (integerp adjusted-value)
+                          (not (assoc adjusted-value labels)))
+                 (push (cons adjusted-value nil) labels)))))
      segment
      dstate)
     (setf (dstate-labels dstate) labels)
@@ -653,16 +726,64 @@
                   (format nil "L~W" max)))))
       (setf (dstate-labels dstate) labels))))
 
+(defun collect-inst-variants (base-name package variants cache)
+  (mapcar
+   (lambda (printer)
+     (destructuring-bind (format-name
+                          (&rest arg-constraints)
+                          &optional (printer :default)
+                          &key (print-name
+                                (without-package-locks (intern base-name package)))
+                               control)
+         printer
+       (declare (type (or symbol string) print-name))
+       (let* ((flavor (cons base-name format-name))
+              (format (format-or-lose format-name))
+              (args (copy-list (format-args format)))
+              (format-length (bytes-to-bits (format-length format))))
+         (dolist (constraint arg-constraints)
+           (destructuring-bind (name . props) constraint
+             (let ((cell (member name args :key #'arg-name))
+                   (arg))
+               (if cell
+                   (setf (car cell) (setf arg (copy-structure (car cell))))
+                   (setf args (nconc args (list (setf arg (%make-arg name))))))
+               (apply #'modify-arg
+                      arg format-length (and props (cons :value props))))))
+         (multiple-value-bind (mask id) (compute-mask-id args)
+           (make-instruction
+                   base-name format-name print-name
+                   (format-length format) mask id
+                   (let ((printer (if (eq printer :default)
+                                      (format-default-printer format)
+                                      printer)))
+                     (when printer
+                       (lambda (chunk inst stream dstate)
+                         (funcall (setf (inst-printer inst)
+                                        (let ((*current-instruction-flavor* flavor))
+                                          (find-printer-fun printer args cache)))
+                                  chunk inst stream dstate))))
+                   (collect-labelish-operands args cache)
+                   (collect-prefiltering-args args cache)
+                   control)))))
+   variants))
+
 ;;; Get the instruction-space, creating it if necessary.
-(defun get-inst-space (&key force)
+(defun get-inst-space (&key (package sb!assem::*backend-instruction-set-package*)
+                            force)
   (let ((ispace *disassem-inst-space*))
     (when (or force (null ispace))
-      (let ((insts nil))
-        (maphash (lambda (name inst-flavs)
-                   (declare (ignore name))
-                   (dolist (flav inst-flavs)
-                     (push flav insts)))
-                 *disassem-insts*)
+      (let ((insts nil)
+            (cache (list (list :printer) (list :prefilter) (list :labeller))))
+        (do-symbols (symbol package)
+          (setq insts (nconc (collect-inst-variants
+                              (string-upcase symbol) package
+                              (get symbol 'instruction-flavors) cache)
+                             insts)))
+        (when force
+          (format t "~&~:{~@(~A~)s: ~D~:^, ~}~%"
+                  (mapcar (lambda (x) (list (car x) (length (cdr x)))) cache)))
+        (setf (cddr cache) nil) ; discard the labeller cache
         (setf ispace (build-inst-space insts)))
       (setf *disassem-inst-space* ispace))
     ispace))
@@ -936,6 +1057,13 @@
   (let ((sap (sb!sys:int-sap address)))
     (lambda () sap)))
 
+(defstruct (source-form-cache (:conc-name sfcache-)
+                              (:copier nil))
+  (debug-source nil :type (or null sb!di:debug-source))
+  (toplevel-form-index -1 :type fixnum)
+  (last-location-retrieved nil :type (or null sb!di:code-location))
+  (last-form-retrieved -1 :type fixnum))
+
 ;;; Return a memory segment located at the system-area-pointer returned by
 ;;; SAP-MAKER and LENGTH bytes long in the disassem-state object DSTATE.
 ;;;
@@ -1016,13 +1144,6 @@
 
 ;;; getting at the source code...
 
-(defstruct (source-form-cache (:conc-name sfcache-)
-                              (:copier nil))
-  (debug-source nil :type (or null sb!di:debug-source))
-  (toplevel-form-index -1 :type fixnum)
-  (last-location-retrieved nil :type (or null sb!di:code-location))
-  (last-form-retrieved -1 :type fixnum))
-
 (defun get-different-source-form (loc context &optional cache)
   (if (and cache
            (eq (sb!di:code-location-debug-source loc)
@@ -1051,8 +1172,11 @@
   (declare (type sb!kernel:code-component code))
   (sb!c::compiled-debug-info-fun-map (sb!kernel:%code-debug-info code)))
 
-(defstruct (location-group (:copier nil))
-  (locations #() :type (vector (or list fixnum))))
+(defstruct (location-group (:copier nil) (:predicate nil))
+  ;; This was (VECTOR (OR LIST FIXNUM)) but that doesn't have any
+  ;; specialization other than T, and the cross-compiler has trouble
+  ;; with (SB!XC:TYPEP #() '(VECTOR (OR LIST FIXNUM)))
+  (locations #() :type simple-vector))
 
 ;;; Return the vector of DEBUG-VARs currently associated with DSTATE.
 (defun dstate-debug-vars (dstate)
@@ -1484,22 +1608,20 @@
     (disassemble-segments segments stream dstate)))
 
 (defun valid-extended-function-designators-for-disassemble-p (thing)
-  (cond ((legal-fun-name-p thing)
-         (compiled-funs-or-lose (fdefinition thing) thing))
-        #!+sb-eval
-        ((sb!eval:interpreted-function-p thing)
-         (compile nil thing))
-        ((typep thing 'sb!pcl::%method-function)
+  (typecase thing
+    ((satisfies legal-fun-name-p)
+     (compiled-funs-or-lose (fdefinition thing) thing))
+    (sb!pcl::%method-function
          ;; in a %METHOD-FUNCTION, the user code is in the fast function, so
          ;; we to disassemble both.
          ;; FIXME: interpreted methods need to be compiled as above.
          (list thing (sb!pcl::%method-function-fast-function thing)))
-        ((functionp thing)
-         thing)
-        ((and (listp thing)
-              (eq (car thing) 'lambda))
-         (compile nil thing))
-        (t nil)))
+    ((or (cons (eql lambda))
+         #!+sb-fasteval sb!interpreter:interpreted-function
+         #!+sb-eval sb!eval:interpreted-function)
+     (compile nil thing))
+    (function thing)
+    (t nil)))
 
 (defun compiled-funs-or-lose (thing &optional (name thing))
   (let ((funs (valid-extended-function-designators-for-disassemble-p thing)))
@@ -1981,6 +2103,8 @@
                (when note
                  (note note dstate))))
         (emit-err-arg)
+        ;; ARM64 encodes the error number in BRK instruction itself
+        #!-arm64
         (emit-err-arg)
         (emit-note (symbol-name (get-internal-error-name errnum)))
         (dolist (sc-offs sc-offsets)
@@ -1993,3 +2117,19 @@
               (emit-note (get-sc-name sc-offs))))))
     (incf (dstate-next-offs dstate)
           adjust)))
+
+;; A prefilter set is a list of vectors specifying bytes to extract
+;; and a function to call on the extracted value(s).
+;; EQUALP lists of vectors can be coalesced, since they're immutable.
+(defun collect-prefiltering-args (args cache)
+  (awhen (remove-if-not #'arg-prefilter args)
+    (let ((repr
+           (mapcar (lambda (arg &aux (bytes (arg-fields arg)))
+                     (coerce (list* (posq arg args)
+                                    (arg-prefilter arg)
+                                    (and bytes (cons (arg-sign-extend-p arg) bytes)))
+                             'vector))
+                   it))
+          (table (assq :prefilter cache)))
+      (or (find repr (cdr table) :test 'equalp)
+          (car (push repr (cdr table)))))))
