@@ -824,7 +824,7 @@ constant shift greater than word length")))
           (t
            (move result number)
            (cond ((< -64 amount 64) ;; XXXX
-                  ;; this code is used both in ASH and ASH-MOD32, so
+                  ;; this code is used both in ASH and ASH-MOD64, so
                   ;; be careful
                   (if (plusp amount)
                       (inst shl result amount)
@@ -1129,6 +1129,20 @@ constant shift greater than word length")))
     (zeroize res)
     DONE))
 
+;; The code on which this was based existed in no less than three varieties,
+;; differing in response to 0 input: produce NIL, -1, or signal an error.
+;; To avoid a thorny issue of proper semantics, this VOP is used only by
+;; %BIT-POSITION which happens to declare zero safety, but always pre-checks
+;; for zero. (the ltn-policy of :fast is actually irrelevant)
+(define-vop (unsigned-word-find-first-bit)
+  (:policy :fast)
+  (:args (arg :scs (unsigned-reg)))
+  (:arg-types unsigned-num)
+  (:results (res :scs (unsigned-reg)))
+  (:result-types unsigned-num)
+  (:generator 1
+    (inst bsf res arg)))
+
 ;; INTEGER-LENGTH is implemented by using the BSR instruction, which
 ;; returns the position of the first 1-bit from the right. And that needs
 ;; to be incremented to get the width of the integer, and BSR doesn't
@@ -1175,61 +1189,6 @@ constant shift greater than word length")))
     (inst or res 1)
     (inst bsr res res)))
 
-(define-vop (unsigned-byte-64-count)
-  (:translate logcount)
-  (:note "inline (unsigned-byte 64) logcount")
-  (:policy :fast-safe)
-  (:args (arg :scs (unsigned-reg) :target result))
-  (:arg-types unsigned-num)
-  (:results (result :scs (unsigned-reg)))
-  (:result-types positive-fixnum)
-  (:temporary (:sc unsigned-reg) temp)
-  (:temporary (:sc unsigned-reg) mask)
-  (:generator 14
-    ;; See the comments below for how the algorithm works. The tricks
-    ;; used can be found for example in AMD's software optimization
-    ;; guide or at "http://www.hackersdelight.org/HDcode/pop.cc" in the
-    ;; function "pop1", for 32-bit words. The extension to 64 bits is
-    ;; straightforward.
-    ;; Calculate 2-bit sums. Note that the value of a two-digit binary
-    ;; number is the sum of the right digit and twice the left digit.
-    ;; Thus we can calculate the sum of the two digits by shifting the
-    ;; left digit to the right position and doing a two-bit subtraction.
-    ;; This subtraction will never create a borrow and thus can be made
-    ;; on all 32 2-digit numbers at once.
-    (move result arg)
-    (move temp arg)
-    (inst shr result 1)
-    (inst mov mask #x5555555555555555)
-    (inst and result mask)
-    (inst sub temp result)
-    ;; Calculate 4-bit sums by straightforward shift, mask and add.
-    ;; Note that we shift the source operand of the MOV and not its
-    ;; destination so that the SHR and the MOV can execute in the same
-    ;; clock cycle.
-    (inst mov result temp)
-    (inst shr temp 2)
-    (inst mov mask #x3333333333333333)
-    (inst and result mask)
-    (inst and temp mask)
-    (inst add result temp)
-    ;; Calculate 8-bit sums. Since each sum is at most 8, which fits
-    ;; into 4 bits, we can apply the mask after the addition, saving one
-    ;; instruction.
-    (inst mov temp result)
-    (inst shr result 4)
-    (inst add result temp)
-    (inst mov mask #x0f0f0f0f0f0f0f0f)
-    (inst and result mask)
-    ;; Add all 8 bytes at once by multiplying with #256r11111111.
-    ;; We need to calculate only the lower 8 bytes of the product.
-    ;; Of these the most significant byte contains the final result.
-    ;; Note that there can be no overflow from one byte to the next
-    ;; as the sum is at most 64 which needs only 7 bits.
-    (inst mov mask #x0101010101010101)
-    (inst imul result mask)
-    (inst shr result 56)))
-
 ;;;; binary conditional VOPs
 
 (define-vop (fast-conditional)
@@ -1376,12 +1335,12 @@ constant shift greater than word length")))
   (define-conditional-vop > :g :a :le :be))
 
 (define-vop (fast-if-eql/signed fast-conditional/signed)
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 6
     (inst cmp x y)))
 
 (define-vop (fast-if-eql-c/signed fast-conditional-c/signed)
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 5
     (cond ((and (sc-is x signed-reg) (zerop y))
            (inst test x x))  ; smaller instruction
@@ -1389,12 +1348,12 @@ constant shift greater than word length")))
            (inst cmp x (constantize y))))))
 
 (define-vop (fast-if-eql/unsigned fast-conditional/unsigned)
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 6
     (inst cmp x y)))
 
 (define-vop (fast-if-eql-c/unsigned fast-conditional-c/unsigned)
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 5
     (cond ((and (sc-is x unsigned-reg) (zerop y))
            (inst test x x))  ; smaller instruction
@@ -1417,7 +1376,7 @@ constant shift greater than word length")))
          (y :scs (any-reg control-stack)))
   (:arg-types tagged-num tagged-num)
   (:note "inline fixnum comparison")
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 4
     (inst cmp x y)))
 
@@ -1435,7 +1394,7 @@ constant shift greater than word length")))
   (:info y)
   (:conditional :e)
   (:policy :fast-safe)
-  (:translate eql)
+  (:translate eql %eql/integer)
   (:generator 2
     (cond ((and (sc-is x any-reg descriptor-reg) (zerop y))
            (inst test x x))  ; smaller instruction
@@ -1447,7 +1406,7 @@ constant shift greater than word length")))
   (:arg-types * (:constant fixnum))
   (:variant-cost 6))
 
-;;;; 32-bit logical operations
+;;;; 64-bit logical operations
 
 ;;; Only the lower 6 bits of the shift amount are significant.
 (define-vop (shift-towards-someplace)
