@@ -219,11 +219,7 @@
                #!+gencgc
                ;; large objects will never be made in a per-thread region
                (and (integerp size)
-                    ;; Kludge: this is supposed to be
-                    ;;  (>= size (extern-alien "large_object_size" long))
-                    ;; but that won't cross-compile. So, a little OAOOM...
-                    (>= size (* 4 (max *backend-page-bytes* gencgc-card-bytes
-                                       gencgc-alloc-granularity)))))
+                    (>= size large-object-size)))
            (allocation-tramp alloc-tn size lowtag))
           (t
            (inst mov temp-reg-tn free-pointer)
@@ -595,7 +591,10 @@ collection."
         ;; BODY is stuffed in a function to preserve the lexical
         ;; environment.
         `(flet ((,wpo () (progn ,@body)))
-           (declare (muffle-conditions compiler-note))
+           ;; The cross-compiler prints either "unknown type: COMPILER-NOTE" at
+           ;; each use of W-P-O prior to 'ir1report' being compiled, or else
+           ;; "could not stack allocate". Kill it with fire :-(
+           (declare (muffle-conditions #+sb-xc compiler-note #-sb-xc t))
            ;; PINS are dx-allocated in case the compiler for some
            ;; unfathomable reason decides to allocate value-cells
            ;; for them -- since we have DX value-cells on x86oid
@@ -611,3 +610,28 @@ collection."
                            `(touch-object ,pin))
                          pins)))))
       `(progn ,@body)))
+
+;;; Emit the most compact form of the test immediate instruction,
+;;; using an 8 bit test when the immediate is only 8 bits and the
+;;; value is one of the four low registers (rax, rbx, rcx, rdx) or the
+;;; control stack.
+(defun emit-optimized-test-inst (x y)
+  (typecase y
+    ((unsigned-byte 7)
+     ;; If we knew that the sign bit would not be tested, this could
+     ;; handle (unsigned-byte 8) constants. But since we don't know,
+     ;; we assume that it's not ok to change the test such that the S flag
+     ;; comes out possibly differently.
+     (let ((offset (tn-offset x)))
+       (cond ((and (sc-is x any-reg descriptor-reg signed-reg unsigned-reg)
+                   (or (= offset rax-offset) (= offset rbx-offset)
+                       (= offset rcx-offset) (= offset rdx-offset)))
+              (inst test (reg-in-size x :byte) y))
+             ((sc-is x control-stack)
+              (inst test (make-ea :byte :base rbp-tn
+                                  :disp (frame-byte-offset offset))
+                    y))
+             (t
+              (inst test x y)))))
+    (t
+     (inst test x y))))
