@@ -221,27 +221,33 @@
                            :foreign))))
 
 (defun allocation-inline (alloc-tn size)
-  (let* ((ok (gen-label)) ;reindent after merging
-        (done (gen-label))
-        #!+(and sb-thread win32)
-        (scratch-tns (loop for my-tn in `(,eax-tn ,ebx-tn ,edx-tn ,ecx-tn)
-                           when (and (not (location= alloc-tn my-tn))
-                                     (or (not (tn-p size))
-                                         (not (location= size my-tn))))
-                             collect my-tn))
-        (tls-prefix #!+sb-thread :fs #!-sb-thread nil)
-        (free-pointer
-         (make-ea :dword :disp
-                  #!+sb-thread (* n-word-bytes thread-alloc-region-slot)
-                  #!-sb-thread (make-fixup "boxed_region" :foreign)
-                  :scale 1)) ; thread->alloc_region.free_pointer
-        (end-addr
-         (make-ea :dword :disp
-                  #!+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
-                  #!-sb-thread (make-fixup "boxed_region" :foreign 4)
-                  :scale 1))   ; thread->alloc_region.end_addr
-        #!+(and sb-thread win32) (scratch-tn (pop scratch-tns))
-        #!+(and sb-thread win32) (swap-tn (pop scratch-tns)))
+  (let* ((ok (gen-label))
+         (done (gen-label))
+         #!+(and sb-thread win32)
+         (scratch-tns (loop for my-tn in `(,eax-tn ,ebx-tn ,edx-tn ,ecx-tn)
+                            when (and (not (location= alloc-tn my-tn))
+                                      (or (not (tn-p size))
+                                          (not (location= size my-tn))))
+                            collect my-tn))
+         (tls-prefix #!+sb-thread :fs)
+         #!+(and sb-thread win32) (scratch-tn (pop scratch-tns))
+         #!+(and sb-thread win32) (swap-tn (pop scratch-tns))
+         (free-pointer
+           ;; thread->alloc_region.free_pointer
+           (make-ea :dword
+                    :base (or #!+(and sb-thread win32)
+                              scratch-tn)
+                    :disp
+                    #!+sb-thread (* n-word-bytes thread-alloc-region-slot)
+                    #!-sb-thread (make-fixup "boxed_region" :foreign)))
+         (end-addr
+            ;; thread->alloc_region.end_addr
+           (make-ea :dword
+                    :base (or #!+(and sb-thread win32)
+                              scratch-tn)
+                    :disp
+                    #!+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
+                    #!-sb-thread (make-fixup "boxed_region" :foreign 4))))
     (unless (and (tn-p size) (location= alloc-tn size))
       (inst mov alloc-tn size))
     #!+(and sb-thread win32)
@@ -251,9 +257,7 @@
       (inst mov scratch-tn
             (make-ea :dword :disp
                      +win32-tib-arbitrary-field-offset+) tls-prefix)
-      (setf (ea-base free-pointer) scratch-tn
-            (ea-base end-addr) scratch-tn
-            tls-prefix nil))
+      (setf tls-prefix nil))
     (inst add alloc-tn free-pointer tls-prefix)
     (inst cmp alloc-tn end-addr tls-prefix)
     (inst jmp :be ok)
@@ -291,7 +295,6 @@
       (inst pop scratch-tn))
     (values)))
 
-
 ;;; Emit code to allocate an object with a size in bytes given by
 ;;; SIZE.  The size may be an integer or a TN. If Inline is a VOP
 ;;; node-var then it is used to make an appropriate speed vs size
@@ -605,7 +608,7 @@ Useful for e.g. foreign calls where another thread may trigger
 collection."
   (if objects
       (let ((pins (make-gensym-list (length objects)))
-            (wpo (gensym "WITH-PINNED-OBJECTS-THUNK")))
+            (wpo (sb!xc:gensym "WITH-PINNED-OBJECTS-THUNK")))
         ;; BODY is stuffed in a function to preserve the lexical
         ;; environment.
         `(flet ((,wpo () (progn ,@body)))
@@ -698,3 +701,27 @@ collection."
        ,@(subst `(make-ea :dword :base ,base :disp ,constant-disp)
                 ea-var
                 (subst nil :maybe-fs body)))))
+
+;;; Emit the most compact form of the test immediate instruction,
+;;; using an 8 bit test when the immediate is only 8 bits and the
+;;; value is one of the four low registers (eax, ebx, ecx, edx) or the
+;;; control stack.
+(defun emit-optimized-test-inst (x y)
+  (typecase y
+    ((unsigned-byte 7)
+     (let ((offset (tn-offset x)))
+       (cond ((and (sc-is x any-reg descriptor-reg)
+                   (or (= offset eax-offset) (= offset ebx-offset)
+                       (= offset ecx-offset) (= offset edx-offset)))
+              (inst test (make-random-tn :kind :normal
+                                         :sc (sc-or-lose 'byte-reg)
+                                         :offset offset)
+                    y))
+             ((sc-is x control-stack)
+              (inst test (make-ea :byte :base ebp-tn
+                                  :disp (frame-byte-offset offset))
+                    y))
+             (t
+              (inst test x y)))))
+    (t
+     (inst test x y))))
